<?php
class ProjectSettings
{
	var $_table;

	var $_pageMode;

	var $_pageType;

	var $_page;

	var $_viewPage = PAGE_VIEW;

	var $_defaultViewPage = PAGE_VIEW;

	var $_editPage = PAGE_EDIT;

	var $_defaultEditPage = PAGE_EDIT;

	var $_tableData = array();

	var $_optionsFile = array();

	/**
	 * _auxTableData is used for all page-related settings such as getDefaultPage, getPageOption etc
	 * The only page so far where table != auxTable is Register
	 * table is <users> there and auxTable is GLOBAL_PAGES
	 */
	var $_auxTable = "";
	var $_auxTableData = array();

	var $_pageOptions = array();

	var $_mastersTableData = array();

	var $_detailsTableData = array();

	var $_dashboardElemPSet = array();

	function __construct($table = "", $pageType = "", $page = "", $pageTable = "")
	{
		if( !$table )
			$table = GLOBAL_PAGES;
		if( !$pageTable )
			$pageTable = $table;
		$this->setTable($table);
		if( $table == $pageTable) {
			$this->_auxTableData = & $this->_tableData;
			$this->_auxTable = $this->_table;
		} else {
			$this->loadAuxTable( $pageTable );
		}
		if( $page ) {
			//	determine the page type to avoid unnecessary permission reading
			//	when creating pSetSearch
			$this->_pageType = $this->getOriginalPageType( $page );
		}
		
		
		if($page && array_key_exists($page, $this->getPageIds())) {
			$this->setPage($page);
			$this->setPageType( $this->getPageType() );
		} else if( $pageType ) {
			$this->_pageType = $pageType;
			$page = $this->getDefaultPage( $pageType );
			if( $page )
			{
				$this->setPage( $page );
			}
		}
		if( $page && !$pageType ) {
			$pageType = $this->getPageType();
		}
		if ( $pageType ) {
			$this->setPageType($pageType);
		}
	}

	function table() {
		return $this->_table;
	}

	function loadPageOptions( $option = "" ) {
		if( $this->_pageOptions )
			return;
		importPageOptions( $this->_auxTable, $this->_page );
		global $page_options;
		$this->_pageOptions = &$page_options[$this->_auxTable][$this->_page];


	}

	function setPage($page) {
		$this->_page = $page;
		//	there is no such page!
		if( array_search( $page, $this->getPageIds() ) === FALSE ) {
			return;
		}
		$this->_optionsFile = getabspath("include/pages/". GetTableURL( $this->_auxTable ) . "_" . $page . ".json");
	}

	function setTable($table)
	{
		$this->_table = $table;
		global $tables_data, $field_labels, $fieldToolTips, $placeHolders, $page_titles, $detailsTablesData, $masterTablesData, $tableEvents, $bSubqueriesSupported;
		if(GetTableURL($table) != "") {
			importTableSettings( $table );
		}

		if(isset($tables_data[$this->_table]))
			$this->_tableData = &$tables_data[$this->_table];

		$this->_mastersTableData = &$masterTablesData[$this->_table];
		$this->_detailsTableData = &$detailsTablesData[$this->_table];

		$tableType = $this->getTableType();

		$this->_editPage = $this->getDefaultEditPageType($tableType);
		$this->_viewPage = $this->getDefaultViewPageType($tableType);
		$this->_defaultEditPage = $this->_editPage;
		$this->_defaultViewPage = $this->_viewPage;
	}

	function loadAuxTable($auxTable)
	{
		$this->_auxTable = $auxTable;
		global $tables_data;
		if(GetTableURL($auxTable) != "") {
			importTableSettings( $auxTable );
		}

		if(isset($tables_data[$this->_auxTable]))
			$this->_auxTableData = &$tables_data[$this->_auxTable];
	}


	function pageName() {
		return $this->_page;
	}

	/**
	 * Return table where the page belongs. <global> for Login, Register page desite table==usersTable
	 */
	function pageTable() {
		return $this->_auxTable;
	}

	function getDefaultViewPageType($tableType)
	{
		if($tableType == PAGE_CHART || $tableType == PAGE_REPORT)
			return $tableType;

		return PAGE_VIEW;
	}

	function getDefaultEditPageType($tableType)
	{
		if($tableType == PAGE_CHART || $tableType == PAGE_REPORT)
			return PAGE_SEARCH;

		return PAGE_EDIT;
	}

	function setPageType($page)
	{
		//	a deeper checking for table and page types compatibility might be added here
		if($this->isPageTypeForView($page))
		{
			$tableType = $this->getTableType();
			if($tableType != "report" && $tableType != "chart"
				&& ($page == PAGE_CHART || $page == PAGE_REPORT))
				$this->_viewPage = PAGE_LIST;
			else
				$this->_viewPage = $page;
			$this->_defaultViewPage = $this->getDefaultViewPageType($tableType);
		}
		if($this->isPageTypeForEdit($page))
		{
			$this->_editPage = $page;
			$this->_defaultEditPage = $this->getDefaultEditPageType($this->getTableType());
		}
	}

	function getDefaultPages() {
		$this->updatePages();
		return $this->getAuxTableData(".defaultPages");
	}

	function getDefaultPage( $type, $pageTable = "" ) {
		$this->updatePages();
		if( isAdminPage( $this->_auxTable ) )
			return $this->_pageType;
		$defPages =& $this->getAuxTableData(".defaultPages");
		$defPage = $defPages[$type];
		if( $defPage )
			return $defPage;
		return null;
	}

	function getPageIds() {
		$this->updatePages();
		$ret = $this->getAuxTableData(".pages");
		if( !is_array( $ret ) )
			return array();
		return $ret;
	}

	function getEditPageType()
	{
		return $this->_editPage;
	}

	function isPageTypeForView($ptype)
	{
		global $pageTypesForView;
		return in_array(strtolower($ptype), $pageTypesForView);
	}

	function isPageTypeForEdit($ptype)
	{
		global $pageTypesForEdit;
		return in_array(strtolower($ptype), $pageTypesForEdit);
	}

	function getTable($table, $page = "")
	{
		return new ProjectSettings($table, $page);
	}

	function getPageTypeByFieldEditFormat($field, $editFormat)
	{
		if(isset($this->_tableData[$field]) && isset($this->_tableData[$field][FORMAT_EDIT]))
		{
			foreach($this->_tableData[$field][FORMAT_EDIT] as $pageType => $editSettings)
			{
				if(isset($editSettings["EditFormat"]) && $editSettings["EditFormat"] == $editFormat)
					return $pageType;
			}
		}
		return "";
	}

	function getTableData($key)
	{
		if(!$this->isExistsTableKey($key))
			return $this->getDefaultValueByKey(substr($key,1));
		return $this->_tableData[$key];
	}

	function getAuxTableData($key)
	{
		if(!$this->isExistsAuxTableKey($key))
			return $this->getDefaultValueByKey(substr($key,1));
		return $this->_auxTableData[$key];
	}

	/**
	 * Specify path to arguments in $key1, $key2 etc
	 * Returns FALSE if not found
	 */
	function getPageOption($key1, $key2 = FALSE, $key3 = FALSE, $key4 = FALSE, $key5 = FALSE )
	{
		$this->loadPageOptions( $key1.$key2 );
		if( !isset( $this->_pageOptions[ $key1 ] ) )
			return FALSE;
		$keys = array( $key1 );
		if( $key2!== FALSE )
			$keys[]= $key2;
		if( $key3!== FALSE )
			$keys[]= $key3;
		if( $key4!== FALSE )
			$keys[]= $key4;
		if( $key5!== FALSE )
			$keys[]= $key5;

		$opt = &$this->_pageOptions;
		foreach( $keys as $k ) {
			if( !is_array( $opt ) )
				return FALSE;
			if( !isset( $opt[ $k ] ) )
				return FALSE;
			$opt = &$opt[$k];
		}
		return $opt;
	}

	/**
	 * Specify path to arguments in $key1, $key2 etc
	 * Returns FALSE if not found
	 */
	function getPageOptionArray( $keys )
	{
		$this->loadPageOptions();
		$opt = &$this->_pageOptions;
		foreach( $keys as $k ) {
			if( !is_array( $opt ) )
				return FALSE;
			if( !isset( $opt[ $k ] ) )
				return FALSE;
			$opt = &$opt[$k];
		}
		return $opt;
	}


	private function getEffectiveEditPage( $field )
	{
		if( $this->isSeparate($field) )
		{
			return $this->_editPage;
		}
		return $this->_defaultEditPage;
	}

	private function getEffectiveViewPage( $field )
	{
		if( $this->isSeparate($field) )
		{
			if( $this->_pageMode == EDIT_INLINE && $this->_viewPage != PAGE_VIEW )
				return PAGE_LIST;
			else if ( $this->_pageMode == LIST_MASTER && $this->_viewPage == PAGE_LIST )
				return PAGE_MASTER_INFO_LIST;
			else if ( $this->_pageMode == LIST_MASTER && $this->_viewPage == PAGE_REPORT )
				return PAGE_MASTER_INFO_REPORT;
			else if ( $this->_pageMode == PRINT_MASTER && $this->_viewPage == PAGE_RPRINT )
				return PAGE_MASTER_INFO_RPRINT;
			else if ( $this->_pageMode == PRINT_MASTER )
				return PAGE_MASTER_INFO_PRINT;

			return $this->_viewPage;
		}
		return $this->_defaultViewPage;
	}
	function getFieldData( $field, $key )
	{
		global $g_settingsType;
		if( $this->getEntityType() == titDASHBOARD )
		{
			return $this->getDashFieldData( $field, $key );
		}

		if(!isset($this->_tableData[$field]))
			return $this->getDefaultValueByKey($key);

		$settingType = $g_settingsType[$key];
		if($settingType == null)
			$settingType = "";

		switch($settingType)
		{
			case SETTING_TYPE_VIEW:
				$viewPage = $this->getEffectiveViewPage( $field );

				if(isset($this->_tableData[$field][FORMAT_VIEW][$viewPage][$key]))
					return $this->_tableData[$field][FORMAT_VIEW][$viewPage][$key];
				break;
			case SETTING_TYPE_EDIT:
				$editPage = $this->getEffectiveEditPage( $field );

				if(isset($this->_tableData[$field][FORMAT_EDIT][$editPage][$key]))
					return $this->_tableData[$field][FORMAT_EDIT][$editPage][$key];
				break;
			default:
				if (isset($this->_tableData[$field][$key]))
					return $this->_tableData[$field][$key];
				break;
		}
		return $this->getDefaultValueByKey($key);
	}

	function setFieldData( $field, $key, $value )
	{
		$oldValue = $this->getFieldData( $field, $key );
		global $g_settingsType;

		$settingType = $g_settingsType[$key];
		if($settingType == null)
			$settingType = "";

		switch($settingType)
		{
			case SETTING_TYPE_VIEW:
				$viewPage = $this->getEffectiveViewPage( $field );
				$this->_tableData[$field][FORMAT_VIEW][$viewPage][$key] = $value;
				break;
			case SETTING_TYPE_EDIT:
				$editPage = $this->getEffectiveEditPage( $field );
				$this->_tableData[$field][FORMAT_EDIT][$editPage][$key] = $value;
				break;
			default:
				$this->_tableData[$field][$key] = $value;
				break;
		}
		return $oldValue;
	}

	/**
	 * getTableName
	 * Returns table name the class is created for
	 */
	function getTableName()
	{
		return $this->_table;
	}

	/**
	 * findField
	 * Returns field name in correct case if the field is found. Empty string otherwise.
	 * @param {string} field name in arbitrary case, original or GoodFieldName result
	 */

	function findField( $f )
	{
		global $field_labels, $mlang_defaultlang;
		//	check exact match
		$fields = $this->getFieldsList();
		if( array_search( $f, $fields ) !== FALSE )
			return $f;

		//	check goodfieldname
		$gTable = GoodFieldName( $this->_table );
		if( isset( $field_labels[ $mlang_defaultlang ][ $f ] ) )
			return $this->getFieldByGoodFieldName( $f );

		//	case-insensitive check
		$f = strtoupper( $f );
		foreach( $fields as $ff )
		{
			if( strtoupper( $ff ) == $f )
				return $ff;

			if( strtoupper( GoodFieldName($ff) ) == $f )
				return $ff;
		}
		return "";

	}

	/**
	 * addCustomExpressionIndex
	 * Add new index to list, for determination of custom expressions position in SQL query
	 * @param {string} table wich contain a lookup field
	 * @param {string} name of lookup field
	 * @param {int} index
	 */
	function addCustomExpressionIndex($mainTable, $mainField, $index)
	{
		if(!$this->isExistsTableKey(".customExpressionIndexes"))
			$this->_tableData[".customExpressionIndexes"] = array();
		if(!isset($this->_tableData[".customExpressionIndexes"][$mainTable]))
			$this->_tableData[".customExpressionIndexes"][$mainTable] = array();
		$this->_tableData[".customExpressionIndexes"][$mainTable][$mainField] = $index;
	}

	/**
	 * getCustomExpressionIndex
	 * Get index of custom expression in SQL field
	 * @param {string} table wich contain a lookup field
	 * @param {string} name of lookup field
	 */
	function getCustomExpressionIndex($mainTable, $mainField)
	{
		if(!$this->isExistsTableKey(".customExpressionIndexes"))
			$this->_tableData[".customExpressionIndexes"] = array();
		if(isset($this->_tableData[".customExpressionIndexes"][$mainTable])
			&& isset($this->_tableData[".customExpressionIndexes"][$mainTable][$mainField]))
			return $this->_tableData[".customExpressionIndexes"][$mainTable][$mainField];

		return FALSE;
	}

	function isExistsTableKey($key)
	{
		if(!isset($this->_tableData[$key]))
			return false;
		return true;
	}

	function isExistsAuxTableKey($key)
	{
		if(!isset($this->_auxTableData[$key]))
			return false;
		return true;
	}

	function isExistsFieldKey($field, $key)
	{
		if(!$this->isExistsTableKey($field))
			return false;
		if(!isset($this->_tableData[$field][$key]))
			return false;
		return true;
	}

	function getDefaultValueByKey($key)
	{
		global $g_defaultOptionValues;
		if(isset($g_defaultOptionValues[$key]))
			return $g_defaultOptionValues[$key];
		return false;
	}

	/**
	 * Returns array of master tables , which are master for current detail table
	 * @return array if success otherwise false
	 */
	function getMasterTablesArr()
	{
		return $this->_mastersTableData;
	}

	/**
	 * Returns array of master keys for passed detailTable
	 *
	 * @param string $dTableName - it's detail data sourse table name,
	 * @return array if success otherwise false
	 */
	function getMasterKeysByDetailTable($dTableName, $default = array())
	{
		if(!$dTableName)
			return $default;
		foreach ($this->_detailsTableData as $dTableDataArr)
		{
			if ($dTableDataArr['dDataSourceTable'] == $dTableName)
				return $dTableDataArr['masterKeys'];
		}
		return $default;
	}

	/**
	 * Returns array of detail tables , which are detail for current master table
	 * @param string $tName - it's data source master table name
	 * @return array if success otherwise false
	 */
	function getDetailTablesArr()
	{
		return $this->_detailsTableData;
	}

	/**
	 * Returns array of detail keys for passed masterTable
	 *
	 * @param string $mTableName - it's master table name,
	 * @param string $tName - it's current (detail) table
	 * @return array if success otherwise default value
	 */
	function getDetailKeysByMasterTable($mTableName = "", $default = array())
	{
		if(!$mTableName)
			return $default;
		foreach($this->_mastersTableData as $mTableDataArr)
		{
			if ($mTableDataArr['mDataSourceTable'] == $mTableName)
				return $mTableDataArr['detailKeys'];
		}
		return $default;
	}

	/**
	 * Returns array of detail keys for passed detailTable
	 *
	 * @param string $dTableName - It's detail data sourse table name
	 * @param string $tName - current(master) table name
	 * @return array if success otherwise false
	 */
	function getDetailKeysByDetailTable($dTableName, $default = array())
	{
		foreach ($this->_detailsTableData as $dTableDataArr)
		{
			if ($dTableDataArr['dDataSourceTable'] == $dTableName)
				return $dTableDataArr['detailKeys'];
		}
		return $default;
	}

	/**
	 * Returns details preview Type
	 *
	 * @param string $dTableName - it's detail data sourse table name,
	 * @param string $tName - current(master) table name
	 * @return array if success otherwise false
	 */
	function getDPType($dTableName)
	{
		if(!$dTableName)
			return false;
		foreach ($this->_detailsTableData as $dTableDataArr)
		{
			if ($dTableDataArr['dDataSourceTable'] == $dTableName)
				return $dTableDataArr['previewOnList'];
		}
		return false;
	}

	function GetFieldByIndex($index)
	{
		foreach($this->_tableData as $key => $value)
		{
			if(!is_array($value))
				continue;
			elseif(!isset($value["Index"]))
				continue;
			if($value["Index"] == $index && $this->getFieldIndex($key))
				return $key;
		}
		return null;
	}

	//	Is field has separate type for editing and viewing
	function isSeparate($field)
	{
		return !!$this->_tableData[$field]["isSeparate"];
	}

	// return field label
	function label($field)
	{
		$result = GetFieldLabel( GoodFieldName($this->_table), GoodFieldName($field) );
		return $result != "" ? $result : $field;
	}

	//	return filename field if any
	//	viewFormat setting
	function getFilenameField($field)
	{
		return $this->getFieldData($field, "Filename");
	}

	//	return hyperlink prefix
	//	viewFormat setting
	function getLinkPrefix($field)
	{
		return $this->getFieldData($field, "LinkPrefix");
	}

	/**
	 * Get hyperlink type
	 */
	function getLinkType($field)
	{
		return $this->getFieldData($field, "hlType");
	}

	/**
	 * Get hyperlink display field for title
	 * @return string field name
	 */
	function getLinkDisplayField($field)
	{
		return $this->getFieldData($field, "hlTitleField");
	}

	function openLinkInNewWindow($field)
	{
		return $this->getFieldData($field, "hlNewWindow");
	}

	function getLinkWordNameType($field)
	{
		return $this->getFieldData($field, "hlLinkWordNameType");
	}

	function getLinkWordText($field)
	{
		return $this->getFieldData($field, "hlLinkWordText");
	}

	//	return database field type
	//	using ADO DataTypeEnum constants
	//	the full list available at:
	//	http://msdn.microsoft.com/library/default.asp?url=/library/en-us/ado270/htm/mdcstdatatypeenum.asp
	function getFieldType($field)
	{
		return $this->getFieldData($field, "FieldType");
	}

	function isAutoincField($field)
	{
		return $this->getFieldData($field, "AutoInc");
	}

	function getOraSequenceName($field)
	{
		return $this->getFieldData($field, "OraSequenceName");
	}

	function getDefaultValue($field, $table = "")
	{
		$tableName = $table ? $table : $this->_table;
		$editPage = $this->_editPage;
		if(!$this->isSeparate($field))
			return;
		return GetDefaultValue($field, $editPage, $tableName);
	}

	function isAutoUpdatable($field)
	{
		return $this->getFieldData($field, "autoUpdatable");
	}

	function getAutoUpdateValue($field)
	{
		$editPage = $this->_editPage;
		if(!$this->isSeparate($field))
			$editPage = $this->getDefaultEditPageType($this->getTableType());;
		return GetAutoUpdateValue($field, $editPage, $this->_table);
	}

	//	return Edit format
	//	editFormats
	function getEditFormat($field)
	{
		return $this->getFieldData($field, "EditFormat");
	}

	//	return View format
	//	viewFormat setting
	function getViewFormat($field)
	{
		return $this->getFieldData($field, "ViewFormat");
	}

	//	show time in datepicker or not
	function dateEditShowTime($field)
	{
		return $this->getFieldData($field, "ShowTime");
	}

	function lookupControlType($field)
	{
		return $this->getFieldData($field, "LCType");
	}

	function lookupListPageId($field)
	{
		return $this->getFieldData($field, "listPageId");
	}
	function lookupAddPageId($field)
	{
		return $this->getFieldData($field, "addPageId");
	}

	function isDeleteAssociatedFile($field)
	{
		return $this->getFieldData($field, "DeleteAssociatedFile");
	}

	//	is Lookup wizard dependent or not
	function useCategory($field)
	{
		return $this->getFieldData($field, "UseCategory");
	}

	//	is Lookup wizard with multiple selection
	function multiSelect($field)
	{
		return $this->getFieldData($field, "Multiselect");
	}

	/**
	 * When a field is edited as single-select lookup wizard
	 * Use case:
	 * if( $pSet->multiSelect() && $pSet->singleSelectLookupEdit() )
	 * multiselect on search, single-select on add/edit. 
	 * In that specific case multiple selected search values 
	 * should be interpreted as:
	 * field=value1 or field=value2 or etc
	 * 
	 */
	function singleSelectLookupEdit( $field )
	{
		$hasLookup = false;
		foreach( $this->_tableData[$field][FORMAT_EDIT] as $pageType => $editFormat ) {
			if( $pageType != "edit" && $pageType != "add" )
				continue;
			if( $editFormat["EditFormat"] != EDIT_FORMAT_LOOKUP_WIZARD )
				continue;
			$hasLookup = true;
			if( $editFormat["Multiselect"] )
				return false;
		}
		return $hasLookup;
	}

	/**
	 * Check whether the field is a true multiselect lookup.
	 * True multiselect has this option on Add or Edit page, not only on Search.
	*/
	function multiSelectLookupEdit( $field )
	{
		$hasLookup = false;
		foreach( $this->_tableData[$field][FORMAT_EDIT] as $pageType => $editFormat ) {
			if( $pageType != "edit" && $pageType != "add" )
				continue;
			if( $editFormat["EditFormat"] != EDIT_FORMAT_LOOKUP_WIZARD )
				continue;
			if( $editFormat["Multiselect"] )
				return true;
		}
		return false;
	}

	// Lookup wizard select size
	function selectSize($field)
	{
		return $this->getFieldData($field, "SelectSize");
	}

	function showThumbnail($field)
	{
		return $this->getFieldData($field, "ShowThumbnail");
	}

	function isImageURL($field)
	{
		return $this->getFieldData($field, "fieldIsImageUrl");
	}

	function showCustomExpr($field)
	{
		return $this->getFieldData($field, "ShowCustomExpr");
	}

	function showFileSize($field)
	{
		return $this->getFieldData($field, "ShowFileSize");
	}

	function showIcon($field)
	{
		return $this->getFieldData($field, "ShowIcon");
	}

	function getImageWidth($field)
	{
		return $this->getFieldData($field, "ImageWidth");
	}

	function getImageHeight($field)
	{
		return $this->getFieldData($field, "ImageHeight");
	}

	function getThumbnailWidth($field)
	{
		return $this->getFieldData($field, "ThumbWidth");
	}

	function getThumbnailHeight($field)
	{
		return $this->getFieldData($field, "ThumbHeight");
	}

	// Get nLookupType for current field
	function getLookupType($field)
	{
		return $this->getFieldData($field, "LookupType");
	}

	//Get lookup table name
	function getLookupTable($field)
	{
		return $this->getFieldData($field, "LookupTable");
	}

	/**
	 *	Returns true if Lookup Where clause is specified as PHP code expression
	 *	@return Boolean
	 */
	function isLookupWhereCode($field)
	{
		return $this->getFieldData($field, "LookupWhereCode");
	}

	function isLookupWhereSet($field)
	{
		if( $this->isLookupWhereCode( $field ) )
			return true;
		return 0 != strlen( $this->getFieldData($field, "LookupWhere") );
	}


	function getLookupWhere($field)
	{
		if( $this->isLookupWhereCode( $field )) {
			$tName = $this->_table;
			if( $this->getEntityType() == titDASHBOARD )
			{
				$dashSearchFields = $this->getDashboardSearchFields();
				if( isset($dashSearchFields[$field]) ) {
					$tName = $dashSearchFields[$field][0]["table"];
					$field = $dashSearchFields[$field][0]["field"];
				}
			}

			return GetLWWhere( $field, $this->getEffectiveEditPage( $field ), $tName );
		}

		return $this->getFieldData($field, "LookupWhere");
	}



	function getNotProjectLookupTableConnId($field)
	{
		return $this->getFieldData($field, "LookupConnId");
	}

	function getConnId()
	{
		$connId = $this->getTableData( ".connId" );
		if( $connId == "" ) {
			return "erp_test3_at_localhost";
		}
		return $connId;
	}

	function getLinkField($field)
	{
		return $this->getFieldData($field, "LinkField");
	}

	function getLWLinkFieldType($field)
	{
		return $this->getFieldData($field, "LinkFieldType");
	}

	function getDisplayField($field)
	{
		return $this->getFieldData($field, "DisplayField");
	}

	function getCustomDisplay($field)
	{
		return $this->getFieldData($field, 'CustomDisplay');
	}

	//	viewFormats
	function NeedEncode($field)
	{
		return $this->getFieldData($field, "NeedEncode");
	}

	/**
	 * Get array of validation for control
	 * return array - of validations
	 */
	function getValidation($field)
	{
		return $this->getFieldData($field, "validateAs");
	}


	/**
	 * Returns array of items for a given field
	 */
	function getFieldItems( $field )
	{
		return $this->getPageOption( "fields", "fieldItems", $field );
	}

	/**
	 * Returns array of group fields
	 */
	function getGroupFields()
	{
		return $this->getPageOption( "dataGrid", "groupFields");
	}

	/**
	 * Check is appear current field on list page
	 * return boolean - true or false
	 */
	function appearOnListPage($field)
	{
		if( array_search( $field, $this->getPageOption("fields", "gridFields") ) !== FALSE )
			return true;
		if( isReport( $this->getEntityType() ) ) {
			return array_search( $field, $this->getReportGroupFields() ) !== FALSE;
		}
		return false;
	}

	/**
	 * Check is appear current field on add page
	 * return boolean - true or false
	 */
	function appearOnAddPage($field)
	{
		return array_search( $field, $this->getPageOption("fields", "gridFields") ) !== FALSE;
	}

	/**
	 * Check is appear current field on inline add
	 * return boolean
	 */
	function appearOnInlineAdd($field)
	{
		return array_search( $field, $this->getInlineAddFields() ) !== FALSE;
		//return $this->getFieldData($field, "bInlineAdd");
	}

	/**
	 * Check is appear current field on edit page
	 * return boolean - true or false
	 */
	function appearOnEditPage($field)
	{
		return array_search( $field, $this->getPageOption("fields", "gridFields") ) !== FALSE;
	}

	/**
	 * Check is appear current field on edit page
	 * return boolean - true or false
	 */
	function appearOnInlineEdit($field)
	{
		return array_search( $field, $this->getInlineEditFields() ) !== FALSE;
		//return $this->getFieldData($field, "bInlineEdit");
	}

	/**
	 * Check is appear current field on edit page
	 * return boolean - true or false
	 */
	function appearOnUpdateSelected($field)
	{
		$updateOnEditFields = $this->getPageOption("fields", "updateOnEditFields");
		if( !$updateOnEditFields )
			return false;

		return array_search( $field, $this->getPageOption("fields", "updateOnEditFields") ) !== FALSE;
		//return $this->getFieldData($field, "bUpdateSelected");
	}

	function appearOnPage($field)
	{
		$gridFields = &$this->getPageOption("fields", "gridFields");
		if( !$gridFields )
			$ret = false;
		else
			$ret = ( array_search( $field, $gridFields ) !== FALSE );
		if( !$ret ) {
			if( $this->getPageType() === 'report' || $this->getPageType() === 'rprint' )
				return array_search( $field, $this->getReportGroupFields() ) !== false;
		}
		return $ret;
	}

	function appearOnSearchPanel($field)
	{
		$fields = &$this->getPageOption("fields", "searchPanelFields");
		if( !$fields )
			return false;

		return array_search( $field, $fields ) !== FALSE;
	}


	function appearAlwaysOnSearchPanel( $field ) {
		$fields  = &$this->getPageOption("listSearch", "alwaysOnPanelFields");
		if( !$fields )
			return false;

		return array_search( $field, $fields ) !== FALSE;

	}

	function getPageFields()
	{
		$fields = $this->getPageOption("fields", "gridFields" );
		if( isReport( $this->getEntityType() ) ) {
			return array_merge( $fields, $this->getReportGroupFields() );
		}
		return $fields;
	}

	/**
	 * Check is appear current field on view page
	 * return boolean - true or false
	 */
	function appearOnViewPage($field)
	{
		return array_search( $field, $this->getPageOption("fields", "gridFields") ) !== FALSE;
	}

	/**
	 * Check is appear current field on print page
	 * return boolean - true or false
	 */
	function appearOnPrinterPage($field)
	{
		return $this->appearOnListPage($field);
	}

	function isVideoUrlField($field)
	{
		return $this->getFieldData($field, "fieldIsVideoUrl");
	}

	function isAbsolute($field)
	{
		return $this->getFieldData($field, "Absolute");
	}

	function getAudioTitleField($field)
	{
		return $this->getFieldData($field, "audioTitleField");
	}

	function getVideoWidth($field)
	{
		return $this->getFieldData($field, "videoWidth");
	}

	function getVideoHeight($field)
	{
		return $this->getFieldData($field, "videoHeight");
	}

	function isRewindEnabled($field)
	{
		return $this->getFieldData($field, "RewindEnabled");
	}

	/**
	 * @param String field
	 * @return Array
	 */
	function getParentFieldsData( $field )
	{
		return $this->getFieldData($field, "categoryFields");
	}

	/**
	 * @param String field
	 * @return Array
	 */
	function getLookupParentFNames( $field )
	{
		$fNames = array();
		foreach( $this->getParentFieldsData( $field ) as $data )
		{
			$fNames[] = $data["main"];
		}
		return $fNames;
	}

	function isLookupUnique($field)
	{
		return $this->getFieldData($field, "LookupUnique");
	}

	function getLookupOrderBy($field)
	{
		return $this->getFieldData($field, "LookupOrderBy");
	}

	function isLookupDesc($field)
	{
		return $this->getFieldData($field, "LookupDesc");
	}

	function getOwnerTable($field)
	{
		return $this->getFieldData($field, "ownerTable");
	}

	function isFieldEncrypted($field)
	{
		return $this->getFieldData($field, "bIsEncrypted");
	}

	function isAllowToAdd($field)
	{
		return $this->getFieldData($field, "AllowToAdd");
	}

	function isSimpleAdd($field)
	{
		return $this->getFieldData($field, "SimpleAdd");
	}

	/**
	 * Get the array containing the autocomplete fields data
	 * basing on page's type and lookup wizard settings
	 * @param String field
	 * @return Array
	 */
	function getAutoCompleteFields($field)
	{
		$editPageType = $this->getEditPageType();

		if( $editPageType == PAGE_REGISTER || $editPageType == PAGE_ADD || $editPageType == PAGE_EDIT && ( $this->isSeparate($field) || $this->isAutoCompleteFieldsOnEdit($field) ) )
		{
			return $this->getFieldData($field, "autoCompleteFields");
		}

		return $this->getDefaultValueByKey("autoCompleteFields");
	}

	function isAutoCompleteFieldsOnEdit($field)
	{
		return $this->getFieldData($field, "autoCompleteFieldsOnEdit");
	}

	function isFreeInput($field)
	{
		return $this->getFieldData($field, "freeInput");
	}

	function getMapData($field)
	{
		return $this->getFieldData($field, "mapData");
	}

	function getFormatTimeAttrs($field)
	{
		return $this->getFieldData($field, "FormatTimeAttrs");
	}

	function getViewAsTimeFormatData( $field ) {
		return $this->getFieldData( $field, "timeFormatData" );
	}
	
	function showDaysInTimeTotals( $field ) {
		$formatData = $this->getViewAsTimeFormatData( $field );
		return $formatData ? $formatData["showDaysInTotals"] : false;
	}	
	
	/**
	 * Check is appear current field on export page
	 * return boolean - true or false
	 */
	function appearOnExportPage($field)
	{
		return $this->appearOnpage($field);
	}

	/**
	 * Return original table name for report or chart
	 */
	function getStrOriginalTableName()
	{
		return $this->getTableData(".strOriginalTableName");
	}

	function getSearchableFields()
	{
		if( $this->getEntityType() == titDASHBOARD ) {
			return $this->getTableData('.allSearchFields');
		}
		return $this->getTableData(".searchableFields");
	}

	function getAllSearchFields()
	{
		return $this->getEntityType() == titDASHBOARD ? $this->getTableData('.allSearchFields') : $this->getPageOption("fields", "searchPanelFields");
	}

	function getAdvSearchFields()
	{
		return $this->getEntityType() == titDASHBOARD ? $this->getAllSearchFields() : $this->getPageOption("fields", "gridFields");;
	}

	function isUseTimeForSearch()
	{
		return $this->getTableData(".isUseTimeForSearch");
	}

	function isUseToolTips()
	{
		return $this->getTableData(".isUseToolTips");
	}

	function isUseVideo()
	{
		return $this->getTableData(".isUseVideo");
	}

	function isUseAudio()
	{
		return $this->getTableData(".isUseAudio");
	}

	function isUseAudioOnDetails()
	{
		for($i = 0; $i < count($this->_detailsTableData); $i++)
		{
			if($this->_detailsTableData[$i]["isUseAudio"])
				return true;
		}
		return false;
	}

	function getTableType()
	{
		return $this->getTableData(".tableType");
	}

	function getShortTableName()
	{
		return GetTableURL( $this->_table );
	}

	function isShowAddInPopup()
	{
		return $this->getPageOption("list", "addInPopup");
	}

	function isShowEditInPopup()
	{
		return $this->getPageOption("list", "editInPopup");
	}

	function isShowViewInPopup()
	{
		return $this->getPageOption("list", "viewInPopup");
	}

	function isResizeColumns()
	{
		return $this->getTableData(".isResizeColumns");
	}

	function isUseAjaxSuggest()
	{
		return $this->getTableData(".isUseAjaxSuggest");
	}

/*
	function getDetailsLinksOnList()
	{
		return $this->getTableData(".detailsLinksOnList");
	}
*/

	function getAllPageFields()
	{
		return array_merge( $this->getPageFields(), $this->getAllSearchFields() );
	}

	function getPanelSearchFields()
	{
		return $this->getPageOption("listSearch", "alwaysOnPanelFields");
	}

	function getSearchPanelOptions()
	{
		return $this->GetTableData(".searchPanelOptions");
	}

	function getGoogleLikeFields()
	{
		return $this->getTableData(".googleLikeFields");
	}

	function getInlineEditFields()
	{
		return $this->getPageOption("fields", "inlineEditFields");
	}

	function getUpdateSelectedFields()
	{
		return $this->getPageOption("fields", "updateOnEditFields");
		//return $this->getTableData(".updateSelectedFields");
	}

	function getExportFields()
	{
		return $this->getPageOption("fields", "exportFields");
	}

	function getImportFields()
	{
		return $this->getPageOption("fields", "gridFields");
	}

	function getEditFields()
	{
		return $this->getPageOption("fields", "gridFields" );
	}

	function getInlineAddFields()
	{
		return $this->getPageOption("fields", "inlineAddFields");
	}

	function getAddFields()
	{
		return $this->getPageOption("fields", "gridFields" );
	}

	function getMasterListFields()
	{
		return $this->getPageOption("fields", "gridFields" );
	}

	function getViewFields()
	{
		return $this->getPageOption("fields", "gridFields" );
	}

	function getPrinterFields()
	{
		return $this->getPageFields();
	}

	function getListFields()
	{
		$fields = $this->getPageOption("fields", "gridFields" );
		if( isReport( $this->getEntityType() ) ) {
			return array_merge( $fields, $this->getReportGroupFields() );
		}
		return $fields;
	}

	function isAddPageEvents()
	{
		return $this->getAuxTableData(".addPageEvents");
	}

	function hasAjaxSnippet()
	{
		return $this->getTableData(".ajaxCodeSnippetAdded");
	}

	function hasButtonsAdded()
	{
		return $this->getPageOption("page", "hasCustomButtons");
	}

	function customButtons()
	{
		return $this->getPageOption("page", "customButtons");
	}

	function isUseFieldsMaps()
	{
		return $this->getTableData(".isUseFieldsMaps");
	}

	function getOrderIndexes()
	{
		return $this->getTableData(".orderindexes");
	}

	function getStrOrderBy()
	{
		return $this->getTableData(".strOrderBy");
	}

	function getSQLQuery()
	{
		$query = $this->getTableData(".sqlquery");
		if($query != null)
		{
			return $query;
		}
		return null;
	}
	/* Dashboard-safe version of getSQLQuery
	*/
	function getSQLQueryByField( $field )
	{
		if( $this->getTableType() == PAGE_DASHBOARD )
		{
			$query = $this->getDashTableData($field, ".sqlquery");
			if($query != null)
			{
				return $query;
			}
			return null;

		}
		else
		{
			$query = $this->getTableData(".sqlquery");
			if($query != null)
			{
				return $query;
			}
			return null;
		}

	}

	/**
	 * Create Thumbnail or not
	 */
	function getCreateThumbnail($field)
	{
		return $this->getFieldData($field, "CreateThumbnail");
	}

	/**
	 * Return Thumbnail prefix
	 */
	function getStrThumbnail($field)
	{
		return $this->getFieldData($field, "StrThumbnail");
	}

	/**
	 * Return Thumbnail prefix
	 */
	function getThumbnailSize($field)
	{
		return $this->getFieldData($field, "ThumbnailSize");
	}

	/**
	 * Resize on upload
	 */
	function getResizeOnUpload($field)
	{
		return $this->getFieldData($field, "ResizeImage");
	}

	/**
	 * True if FileField must work in old single-file mode
	 */
	function isBasicUploadUsed($field)
	{
		return $this->getFieldData($field, "CompatibilityMode");
	}

	/**
	 * True if file in FileField must be uploaded immediately after choosing or dropping
	 */
	function isAutoUpload($field)
	{
		return $this->getFieldData($field, "autoUpload");
	}

	/**
	 * Get size to reduce image after upload
	 */
	function getNewImageSize($field)
	{
		return $this->getFieldData($field, "NewSize");
	}

	function getAcceptFileTypes($field)
	{
		return $this->getFieldData($field, "acceptFileTypes");
	}

	function getAcceptFileTypesHtml($field)
	{
		return $this->getFieldData($field, "acceptFileTypesHtml");
	}

	/**
	 * Get maximum allowed size for uploaded files
	 */
	function getMaxFileSize($field)
	{
		return $this->getFieldData($field, "maxFileSize");
	}

	/**
	 * Get maximum allowed size for all uploaded files per field
	 */
	function getMaxTotalFilesSize($field)
	{
		return $this->getFieldData($field, "maxTotalFilesSize");
	}

	/**
	 * Get maximum allowed number of uploaded files
	 */
	function getMaxNumberOfFiles($field)
	{
		return $this->getFieldData($field, "maxNumberOfFiles");
	}

	/**
	 * Get maximum allowed width of uploaded image
	 */
	function getMaxImageWidth($field)
	{
		return $this->getFieldData($field, "maxImageWidth");
	}

	/**
	 * Get maximum allowed heiht of uploaded image
	 */
	function getMaxImageHeight($field)
	{
		return $this->getFieldData($field, "maxImageHeight");
	}

	/**
	 * Get size to reduce image after upload
	 */
	function getStrFilename($field)
	{
		return $this->getFieldData($field, "strFilename");
	}

	/**
	 * Return height of text area
	 * EditFormat setting
	 */
	function getNRows($field)
	{
		return $this->getFieldData($field, "nRows");
	}

	/**
	 * Return width of text area
	 * EditFormat setting
	 */
	function getNCols($field)
	{
		return $this->getFieldData($field, "nCols");
	}

	/**
	 * Return original table name
	 */
	function getOriginalTableName()
	{
		$result = $this->getTableData(".OriginalTable");
		return $result != "" ? $result : $this->_table;
	}

	/**
	 * Return list of key fields
	 */
	function getTableKeys()
	{
		return $this->getTableData(".Keys");
	}

	function isLargeTextTruncationSet($field)
	{
		return $this->getFieldData($field, "truncateText");
	}

	/**
	 * Return number of chars to show before "More..." link
	 */
	function getNumberOfChars($field)
	{
		return $this->getFieldData($field, "NumberOfChars");
	}

	/**
	 * Check if the field is an sql expression
	 */
	function isSQLExpression($field)
	{
		return $this->getFieldData($field, "isSQLExpression");
	}

	/**
	 * Get full field name
	 */
	function getFullFieldName($field)
	{
		return $this->getFieldData($field, "FullName");
	}

	/**
	 * set full field name
	 */
	function setFullFieldName($field, $value)
	{
		return $this->setFieldData($field, "FullName", $value);
	}


	/**
	 * Return table Owner ID field
	 */
	function getTableOwnerID()
	{
		return $this->getTableData(".OwnerID");
	}

	/**
	 * Is field marked as required
	 * EditFormat setting
	 */
	function isRequired($field)
	{
		return $this->getFieldData($field, "IsRequired");
	}

	function insertNull($field)
	{
		return $this->getFieldData($field, "insertNull");
	}

	/**
	 * IS use Rich Text Editor or not
	 */
	function isUseRTE($field)
	{
		return $this->getFieldData($field, "UseRTE");
	}

	/**
	 * Is use Rich Text Editor BASIC or not
	 */
	function isUseRTEBasic($field)
	{
		global $isUseRTEBasic;
		return $this->isUseRTE($field) && $isUseRTEBasic;
	}

	/**
	 * Is use Rich Text Editor FCK or not
	 */
	function isUseRTEFCK($field)
	{
		global $isUseRTECK;
		return $this->isUseRTE($field) && $isUseRTECK;
	}

	/**
	 * check if RTE CKEditor version 5 is used in project
	 */
	function isUseRTECKNew($field)
	{
		global $isUseRTECKNew;
		return $this->isUseRTE($field) && $isUseRTECKNew;
	}	
	
	/**
	 * Is use Rich Text Editor INNOVA or not
	 */
	function isUseRTEInnova($field)
	{
		global $isUseRTEInnova;
		return $this->isUseRTE($field) && $isUseRTEInnova;
	}

	/**
	 * Add timestamp to filename when files uploading or not
	 * EditFormat setting
	 */
	function isUseTimestamp($field)
	{
		return $this->getFieldData($field, "UseTimestamp");
	}

	function getFieldIndex($field)
	{
		return $this->getFieldData($field, "Index");
	}

	function getEntityType()
	{
		return $this->getTableData(".entityType");
	}


	/**
	 * Return Date field edit type
	 */
	function getDateEditType($field)
	{
		return $this->getFieldData($field, "DateEditType");
	}

	function getHTML5InputType($field)
	{
		return $this->getFieldData($field, "HTML5InuptType");
	}

	/**
	 * Return text edit parameters
	 */
	function getEditParams($field)
	{
		return $this->getFieldData($field, "EditParams");
	}


	function getControlWidth($field)
	{
		return $this->getFieldData($field, "controlWidth");
	}

	/**
	 * Check whether field is viewable
	 */
	function checkFieldPermissions($field)
	{
		return $this->appearOnPage( $field );
	}

	function getTableOwnerIdField()
	{
		return $this->getTableData(".mainTableOwnerID");
	}

	function isHorizontalLookup($field)
	{
		return $this->getFieldData($field, "HorizontalLookup");
	}

	function isDecimalDigits($field)
	{
		return $this->getFieldData($field, "DecimalDigits");
	}

	function getLookupValues($field)
	{
		return $this->getFieldData($field, "LookupValues");
	}

	function hasEditPage()
	{
		return !!$this->getDefaultPage( "edit" );
	}

	function hasAddPage()
	{
		return !!$this->getDefaultPage( "add" );
	}

	function hasListPage()
	{
		return !!$this->getDefaultPage( "list" );
	}

	function hasImportPage()
	{
		return !!$this->getDefaultPage( "import" );
	}

	function hasInlineEdit()
	{
		return $this->getPageOption("list", "inlineEdit");
	}

	function hasUpdateSelected()
	{
		return $this->getPageOption("list", "updateSelected");
	}

	function updateSelectedButtons()
	{
		//return $this->getPageOption("edit", "updateSelectedButtons");
		$data = $this->labeledButtons();
		return $data['update_records'];
	}

	function activatonMessages()
	{
		$data = $this->labeledButtons();
		if( !is_array( $data['register_activate_message'] ) )
			return array();
		return $data['register_activate_message'];
	}

	function labeledButtons()
	{
		return $this->getPageOption("page", "labeledButtons");
	}

	function printPagesLabelsData()
	{
		$data = $this->labeledButtons();
		return $data['print_pages'];
	}

	function detailsFoundLabelsData()
	{
		$data = $this->labeledButtons();
		return $data['details_found'];
	}


	function hasSortByDropdown()
	{
		return $this->getPageOption("list", "sortDropdown");
	}

	function getSortControlSettingsJSONString()
	{
		return $this->getTableData(".strSortControlSettingsJSON");
	}
	function getClickActionJSONString()
	{
		return $this->getTableData(".strClickActionJSON");
	}

	function hasCopyPage()
	{
		return true;
		//return $this->getPageOption("pageLinks", "copy");
	}
	function hasViewPage()
	{
		return !!$this->getDefaultPage("view");
	}
	function hasExportPage()
	{
		return !!$this->getDefaultPage("export");
	}
	function hasPrintPage()
	{
		return !!$this->getDefaultPage("print") || !!$this->getDefaultPage("rprint");
	}
	function hasDelete()
	{
		return $this->getPageOption("list", "delete");
	}
	function getTotalsFields()
	{
		$ret = array();
		foreach( $this->getPageOption('totals') as $field => $totals ) {
			if( $totals /* ?? */ && $totals["totalsType"] ) {
				$ret[] = array(
					"fName" => $field,
					"numRows" => 0,
					"totalsType" => $totals["totalsType"],
					"viewFormat" => $this->getViewFormat( $field )

				);
			}
		}
		return $ret;
	}

	function getExportTxtFormattingType()
	{
		return $this->getPageOption("export","format");
	}

	function getExportDelimiter()
	{
		return $this->getPageOption("export","delimiter");
	}

	function chekcExportDelimiterSelection()
	{
		return $this->getPageOption("export","selectDelimiter");
	}

	function checkExportFieldsSelection()
	{
		return $this->getPageOption("export","selectFields");
	}

	function exportFileTypes()
	{
		return $this->getPageOption("export", "exportFileTypes");
	}

	function getLoginFormType()
	{
		return $this->getPageOption("loginForm", "loginForm");
	}

	function getAdvancedSecurityType()
	{
		if( !GetGlobalData("createLoginPage",null) || (GetGlobalData("nLoginMethod", null) != SECURITY_TABLE && GetGlobalData("nLoginMethod", null) != SECURITY_AD ) )
			return ADVSECURITY_ALL;
		return $this->getTableData(".nSecOptions");
	}
	function displayLoading()
	{
		return $this->getTableData(".isDisplayLoading");
	}
	function getRecordsPerPageArray()
	{
		return $this->getTableData(".arrRecsPerPage");
	}
	function getGroupsPerPageArray()
	{
		return $this->getTableData(".arrGroupsPerPage");
	}

	function isReportWithGroups()
	{
		return !!$this->getPageOption("newreport", "reportInfo", "groupFields" );
		//return $this->getTableData(".reportGroupFields");
	}

	function isCrossTabReport()
	{
		return $this->getPageOption("newreport", "reportInfo", "crosstab" );
		//return $this->getTableData(".crossTabReport");
	}

	function getReportGroupFields()
	{
		$ret = array();
		foreach( $this->getPageOption("newreport", "reportInfo", "groupFields" ) as $g ) {
			$ret[] = $g["field"];
		}
		return $ret;
		//return $this->getTableData(".reportGroupFieldsList");
	}

	function getReportGroupFieldsData()
	{
		$ret = array();
		foreach( $this->getPageOption("newreport", "reportInfo", "groupFields" ) as $idx => $g ) {
			$gdata = array();
			$gdata["strGroupField"] = $g["field"];
			$gdata["groupInterval"] = $g["interval"];
			$gdata["groupOrder"] = $idx + 1;
			$gdata["showGroupSummary"] = $g["summary"];
			$gdata["crossTabAxis"] = $g["axis"];
			$ret[] = $gdata;
		}
		return $ret;
//		return $this->getTableData(".reportGroupFieldsData");
	}

	function reportHasHorizontalSummary()
	{
		return $this->getPageOption("newreport", "reportInfo", "horizSummary" );
	}

	function reportHasVerticalSummary()
	{
		return $this->getPageOption("newreport", "reportInfo", "vertSummary" );
	}

	function reportHasPageSummary()
	{
		return $this->getPageOption("newreport", "reportInfo", "pageSummary" );
	}

	function reportHasGlobalSummary()
	{
		return $this->getPageOption("newreport", "reportInfo", "globalSummary" );
	}

	function getReportLayout()
	{
		//return  $this->getTableData(".reportLayout");
		$rLayout =  $this->getPageOption("newreport", "reportInfo", "layout" );
		if( $rLayout === 'stepped' ) {
			return REPORT_STEPPED;
		} 
		else if( $rLayout === 'align' ) {
			return REPORT_ALIGN;
		}
		else if( $rLayout === 'outline' ) {
			return REPORT_OUTLINE;
		}
		else if( $rLayout === 'block' ) {
			return REPORT_BLOCK;
		} else {
			return REPORT_TABULAR;
		}
	}

	function isGroupSummaryCountShown()
	{
		foreach( $this->getPageOption("newreport", "reportInfo", "groupFields" ) as $g ) {
			if( $g["summary"] ) {
				return true;
			}
		}
		return false;
	}

	function reportDetailsShown()
	{
		return $this->getPageOption("newreport", "reportInfo", "showData" );
	}

	function reportTotalFieldsExist()
	{
		foreach( $this->getPageOption("newreport", "reportInfo", "fields" ) as $f ) {
			if( $f["sum"] || $f["min"] || $f["max"] || $f["avg"] ) {
				return true;
			}
		}
		return false;
	}

	/**
	 * boolean
	 * True if the field has 'max' total in report
	 */
	function reportFieldInfo( $field ) {
		foreach( $this->getPageOption("newreport", "reportInfo", "fields" ) as $f ) {
			if( $f["field"] === $field ) {
				return $f;
			}
		}
		return null;
	}

	function noRecordsOnFirstPage()
	{
		return $this->getTableData(".noRecordsFirstPage");
	}

	function isViewPagePDF()
	{
		return $this->getTableData(".isViewPagePDF");
	}

	function isLandscapeViewPDFOrientation()
	{
		return $this->getTableData(".isLandscapeViewPDFOrientation");
	}

	function isViewPagePDFFitToPage()
	{
		return $this->getTableData(".isViewPagePDFFitToPage");
	}

	function getViewPagePDFScale()
	{
		return $this->getTableData(".nViewPagePDFScale");
	}

	function isLandscapePrinterPagePDFOrientation()
	{
		return $this->getTableData(".isLandscapePrinterPagePDFOrientation");
	}

	function isPrinterPagePDFFitToPage()
	{
		return $this->getTableData(".isPrinterPagePDFFitToPage");
	}

	function getPrinterPagePDFScale()
	{
		return $this->getTableData(".nPrinterPagePDFScale");
	}

	function isPrinterPageFitToPage()
	{
		return $this->getTableData(".isPrinterPageFitToPage");
	}

	function getPrinterPageScale()
	{
		return $this->getTableData(".nPrinterPageScale");
	}

	function getPrinterSplitRecords()
	{
		return $this->getTableData(".nPrinterSplitRecords");
	}

	function getPrinterPDFSplitRecords()
	{
		return $this->getTableData(".nPrinterPDFSplitRecords");
	}

	function isPrinterPagePDF()
	{
		return $this->getPageOption("pdf", "pdfView");
	}

	function hasCaptcha() {
		return $this->getPageOption( "captcha", "captcha" );
	}

	function hasBreadcrumb() {
		return $this->getPageOption( "misc", "breadcrumb" );
	}

	function isSearchRequiredForFiltering()
	{
		return $this->getTableData(".searchIsRequiredForFilters");
	}
	function warnLeavingPages()
	{
		return $this->getTableData(".warnLeavingPages");
	}

	function hideEmptyViewFields()
	{
		return $this->getTableData(".hideEmptyFieldsOnView");
	}

	function getInitialPageSize()
	{
		if( isReport( $this->getEntityType() ) ) {
			if( $this->isReportWithGroups() ) {
				return $this->getTableData(".pageSizeGroups");
			} else {
				return $this->getTableData(".pageSizeRecords");
			}

		}
		return $this->getTableData(".pageSize");
	}

	function getRecordsPerRowList()
	{
		return $this->getPageOption( "page", "recsPerRow" );
	}

	function getRecordsPerRowPrint()
	{
		return $this->getPageOption( "page", "recsPerRow" );
	}

	function getRecordsLimit()
	{
		return $this->getTableData(".recsLimit");
	}

	function useMoveNext()
	{
		return $this->getPageOption( "misc", "nextPrev" );
	}

	function highlightRows()
	{
		return $this->getTableData(".rowHighlite");
	}

	function hasInlineAdd()
	{
		return $this->getpageOption("list", "inlineAdd");
	}

	function getListGridLayout()
	{
		return $this->getPageOption("page", "gridType");
	}

	function getPrintGridLayout()
	{
		return $this->getPageOption("page", "gridType");
	}

	/*
	function getReportPrintLayout()
	{
		return $this->getTableData(".printReportLayout");
	}
	*/

	function getPrinterPageOrientation()
	{
		return $this->getTableData(".printerPageOrientation");
	}


	function getReportPrintGroupsPerPage()
	{
		if( $this->isReportWithGroups() )
			return $this->getTableData(".reportPrintGroupsPerPage");
		else 
			return $this->getTableData(".reportPrintRecordsPerPage");

	}

	function getReportPrintPDFGroupsPerPage()
	{
		return $this->getTableData(".reportPrintPDFGroupsPerPage");
	}

	function ajaxBasedListPage()
	{
		return $this->getTableData(".listAjax");
	}

	function isMultistepped()
	{
		return $this->getPageOption("page", "multiStep");
	}

	function hasVerticalBar()
	{
		return $this->getPageOption("page", "verticalBar");
	}


	/**
	 * Returns array of tabs, which use on list/report/chart page
	 * @return array
	 */
	function getGridTabs()
	{
		return $this->getTableData(".arrGridTabs");
	}


	function highlightSearchResults()
	{
		return $this->getTableData(".highlightSearchResults");
	}


	function getFieldsList()
	{
		if(is_null($this->_tableData))
			return array();
		$t = array_keys($this->_tableData);
		$arr = array();
		foreach($t as $f)
			if(substr($f,0,1)!=".")
				$arr[] = $f;
		return $arr;
	}

	function getFieldCount() {
		return count( $this->getFieldsList() );
	}


	function getBinaryFieldsIndices()
	{
		$fields = $this->getFieldsList();
		$out = array();
		foreach($fields as $idx => $f)
		{
			if(IsBinaryType($this->getFieldType($f)))
				$out[] = $idx + 1;
		}
		return $out;
	}

	function getNBFieldsList()
	{
		$t = $this->getFieldsList();
		$arr = array();
		foreach($t as $f)
			if(!IsBinaryType($this->getFieldType($f)))
				$arr[] = $f;
		return $arr;
	}

	/**
	 * Get field by good field name
	 * @param string	$field The good field name
	 * @return string
	 */
	function getFieldByGoodFieldName($field)
	{
		foreach($this->_tableData as $key => $value)
		{
			if( is_array( $value ) && count($value) > 1)
			{
				if($value["GoodName"] == $field)
					return $key;
			}
		}
		return "";
	}

	/**
	 * getUploadFolder
	 * Return inputed value or calculated path for upload folder
	 * @param {string} field name
	 * @param {array} file info (name, type, size)
	 */
	function getUploadFolder($field, $fileData = array())
	{
		if($this->isUploadCodeExpression($field))
			$path = GetUploadFolderExpression($field, $fileData, $this->_table );
		else
			$path = $this->getFieldData($field, "UploadFolder");
				if(strlen($path) && substr($path,strlen($path)-1) != "/")
			$path.="/";
		return $path;
	}

	function isMakeDirectoryNeeded($field)
	{
		return $this->isUploadCodeExpression($field) || !$this->isAbsolute($field);
	}

	function getFinalUploadFolder($field, $fileData = array())
	{
		if($this->isAbsolute($field))
			$path = $this->getUploadFolder($field, $fileData);
		else
			$path = getabspath($this->getUploadFolder($field, $fileData));
				if(strlen($path) && substr($path,strlen($path)-1) != "/")
			$path.="/";
		return $path;
	}

	function isUploadCodeExpression($field)
	{
		return $this->getFieldData($field, "UploadCodeExpression");
	}

	function &getQueryObject()
	{
		$queryObj = $this->getSQLQuery();
		return $queryObj;
	}

	function getListOfFieldsByExprType($needaggregate)
	{
		$query = &$this->getSQLQuery();
		if( !$query ) {
			return array();
		}
		$fields = $this->getFieldsList();
		$out = array();
		foreach($fields as $idx=>$f)
		{
			$aggr = $query->IsAggrFuncField($idx);
			if($needaggregate && $aggr || !$needaggregate && !$aggr)
				$out[] = $f;
		}
		return $out;
	}

	function isAggregateField($field)
	{
		$query = &$this->getSQLQuery();
		$idx = $this->getFieldIndex($field) - 1;
		return $query->IsAggrFuncField($idx);
	}

	/**
	 * Check if searching is case insensitive for the table
	 * @return Boolean
	 */
	function getNCSearch()
	{
		return $this->getTableData(".NCSearch");
	}

	function getChartType()
	{
		return $this->getTableData(".chartType");
	}

	function getChartRefreshTime()
	{
		return $this->getTableData(".ChartRefreshTime");
	}

	function getChartXml()
	{
		return $this->getTableData(".chartXml");
	}

	function auditEnabled()
	{
		return $this->getTableData(".audit");
	}

	function isSearchSavingEnabled()
	{
		return $this->getPageOption( "listSearch", "searchSaving" );
	}

	function isAllowShowHideFields()
	{
		if ( $this->getScrollGridBody() )
			return false;

		return $this->getPageOption( "list", "showHideFields" );
		//return $this->getTableData(".allowShowHideFields");
	}

	function isAllowFieldsReordering()
	{
		if ( $this->getScrollGridBody() || $this->getRecordsPerRowList() > 1)
			return false;

		return $this->getPageOption( "list", "reorderFields" );
	}

	function lockingEnabled()
	{
		return $this->getTableData(".locking");
	}

	function hasEncryptedFields()
	{
		return $this->getTableData(".hasEncryptedFields");
	}

	function showSearchPanel()
	{
		//return $this->getTableData(".showSearchPanel");
		return $this->getPageOption( "listSearch", "searchPanel" );
	}

	function isFlexibleSearch()
	{
		// return $this->getTableData(".flexibleSearch");
		return !$this->getPageOption( "listSearch", "fixedSearchPanel" );
	}

	function getSearchRequiredFields()
	{
		return $this->getPageOption("fields", "searchRequiredFields");
	}

	function showSimpleSearchOptions()
	{
//		return $this->getTableData(".showSimpleSearchOptions");
		return $this->getPageOption("listSearch", "simpleSearchOptions");
	}

	function getFieldsToHideIfEmpty()
	{
		return $this->getPageOption("fields", "hideEmptyFields");
	}

	function getFilterFields()
	{
		$ret = $this->getPageOption("fields", "filterFields");
		if( !$ret )
			return array();
		return $ret;
	}

	function getFilterFieldFormat($field)
	{
		return $this->getFieldData($field, "filterFormat");
	}

	function getFilterFieldTotal($field)
	{
		return $this->getFieldData($field, "filterTotals");
	}

	function showWithNoRecords($field)
	{
		return $this->getFieldData($field, "showWithNoRecords");
	}

	function getFilterSortValueType($field)
	{
		return $this->getFieldData($field, "sortValueType");
	}

	function isFilterSortOrderDescending($field)
	{
		return $this->getFieldData($field, "descendingOrder");
	}

	function getNumberOfVisibleFilterItems($field)
	{
		return $this->getFieldData($field, "numberOfVisibleItems");
	}

	function getFilterByInterval($field)
	{
		return $this->getFieldData($field, "filterBy");
	}

	function getParentFilterName($field)
	{
		return $this->getFieldData($field, "parentFilterField");
	}

	function getFilterIntervals($field)
	{
		return $this->getFieldData($field, "filterIntervals");
	}

	function showCollapsed($field)
	{
		return $this->getFieldData($field, "showCollapsed");
	}

	function getFilterIntervalDatabyIndex($field, $idx)
	{
		$intervalData = array();

		$filterIntervalsData = $this->getFilterIntervals($field);
		foreach($filterIntervalsData as $interval)
		{
			if($interval["index"] == $idx)
			{
				$intervalData = $interval;
				break;
			}
		}

		return $intervalData;
	}

	function getFilterTotalsField($field)
	{
		return $this->getFieldData($field, "filterTotalFields");
	}

	function getFilterFiledMultiSelect($field)
	{
		return $this->getFieldData($field, "filterMultiSelect");
	}
	
	function getBooleanFilterMessageData( $field, $checked ) {
		if( $checked ) {
			return array (
					"text" => $this->getFieldData($field, "filterCheckedMessageText"),
					"type" => $this->getFieldData($field, "filterCheckedMessageType")
				);
		}
		return array (
				"text" => $this->getFieldData($field, "filterUncheckedMessageText"),
				"type" => $this->getFieldData($field, "filterUncheckedMessageType")
			);
	}
	

	function getFilterStepType($field)
	{
		return $this->getFieldData($field, "filterSliderStepType");
	}

	function getFilterStepValue($field)
	{
		return $this->getFieldData($field, "filterSliderStepValue");
	}

	function getFilterKnobsType($field)
	{
		return $this->getFieldData($field, "filterKnobsType");
	}

	function isFilterApplyBtnSet($field)
	{
		return $this->getFieldData($field, "filterApplyBtn");
	}

	function isCaseInsensitiveUsername()
	{
		global $caseInsensitiveUsername;
		return $caseInsensitiveUsername;
	}

	/**
	 * Returns true if Users table is included into the project ( and has ist own SQL string )
	 * @return boolean
	 */
	function usersTableInProject()
	{
		return GetGlobalData("usersTableInProject",false);
	}


	function getStrField($field)
	{
		return $this->getFieldData($field, "strField");
	}

	function getSourceSingle( $field )
	{
		return $this->getFieldData($field, "sourceSingle");
	}

	function getFieldSource( $field, $listRequest )
	{
		return $listRequest 
			? $this->getFieldData( $field, "strField" )
			: $this->getFieldData( $field, "sourceSingle" );
	}

	function getScrollGridBody()
	{
		return $this->getTableData(".scrollGridBody");
	}

	/**
	 * Is 'UpdateLatLng' ticked for the table
	 */
	function isUpdateLatLng()
	{
		return $this->getTableData(".geocodingEnabled");
	}

	/**
	 * get geocoding data for the table
	 */
	function getGeocodingData()
	{
		return $this->getTableData(".geocodingData");
	}

	function allowDuplicateValues($field)
	{
		return !$this->getFieldData($field, "denyDuplicates");
	}

	function getDashFieldData($field, $key)
	{
		$dashSearchFields = $this->getDashboardSearchFields();
		$dfield = $dashSearchFields[ $field ];
		if( $dfield )
			$table = $dfield[0]["table"];
		if( !$dfield || !$table)
			return $this->getDefaultValueByKey( $key );

		if( !$this->_dashboardElemPSet[ $table ] )
			$this->_dashboardElemPSet[ $table ] = new ProjectSettings( $table, $this->_editPage );

		return $this->_dashboardElemPSet[ $table ]->getFieldData( $dfield[0]["field"], $key );
	}

	function getDashTableData($field, $key)
	{
		$dashSearchFields = $this->getDashboardSearchFields();
		$tableSettings = new ProjectSettings($dashSearchFields[$field][0]["table"], $this->_editPage);
		return $tableSettings->getTableData( $key );
	}


	/**
	 * It returns an empty array if the 'Select all' (search options) is ticked
	 */
	function getSearchOptionsList($field)
	{
		return $this->getFieldData($field, "searchOptionsList");
	}


	function getDefaultSearchOption($field)
	{
		$defaultOpt = $this->getFieldData($field, "defaultSearchOption");

		if( !$defaultOpt )
		{
			$searchOptionsList = $this->getSearchOptionsList($field);
			if( count($searchOptionsList) )
				$defaultOpt = $searchOptionsList[0];
		}
		return $defaultOpt;
	}

	/**
	 * Get status show list of thumbnails
	 * @param string $field field name
	 * @return boolean
	 */
	function showListOfThumbnails($field)
	{
		return $this->getFieldData($field, "ShowListOfThumbnails");
	}

	static function isMenuTreelike( $menuName )
	{
		global $menuTreelikeFlags;
		return $menuTreelikeFlags[$menuName];
	}

	function setPageMode($pageMode)
	{
		$this->_pageMode = $pageMode;
	}

	function editPageHasDenyDuplicatesFields()
	{
		foreach($this->getEditFields() as $fieldName)
		{
			if( !$this->allowDuplicateValues($fieldName) )
			{
				return true;
			}
		}
		return false;
	}

	/**
	 * Get a field's RTE type for RTE fields
	 * and the empty string for others
	 * @param {String} $field 	A field's name
	 * @return {String}
	 */
	function getRTEType($field)
	{
		if($this->isUseRTEBasic($field))
		{
			return "RTE";
		}
		
		if( $this->isUseRTECKNew($field) )
		{
			return "RTECK_NEW";
		}
		
		if($this->isUseRTEFCK($field))
		{
			return "RTECK";
		}
		if($this->isUseRTEInnova($field))
		{
			return "RTEINNOVA";
		}
		return "";
	}

	/**
	 * Get the list of the fields that must be hidden on the List page on a particular device
	 * @param {Number} Device code : 1-4
	 * @return {Array} array in the form of: $arr[$field]=true, where the $field must be hidden.
	 */
	function getHiddenFields($device)
	{
		$list = $this->getTableData( ".hideMobileList" );
		if( isset( $list[$device] ) )
			return $list[$device];
		return array();
	}

	/**
	 * Get the list of the fields that must be hidden on the List page on a particular device
	 * @param {Number} Device code : 1-4
	 * @return {Array} array in the form of: $arr[$goodfieldName]=true, where the $field must be hidden.
	 */
	function getHiddenGoodNameFields($device)
	{
		$hGoodFields = array();
		$hFields = $this->getHiddenFields($device);
		foreach ( $hFields as $field => $isShow ) {
			$hGoodFields[GoodFieldName($field)] = $isShow;
		}

		return $hGoodFields;
	}

	/**
	 * Checks if the 'columns By Device' is enabled
	 * @return {Boolean}
	 */
	function columnsByDeviceEnabled()
	{
		$list = $this->getTableData( ".hideMobileList" );
		foreach( $list as $d => $v )
		{
			if( $v )
				return true;
		}
		return false;
	}


	/**
	 * Build CSS media clause for given device code.
	 * Thanks to mobile device vendors, there are plenty of weird heuristics here, take it easy.
	 */
	static function getDeviceMediaClause($device)
	{
		if( $device == DESKTOP )
		{
			// width >= 1281

			//	Desktop. We don't care about prehistoric VGA displays
			return "@media (min-device-width: 1281px)";
		}
		else if( $device == TABLET_10_IN )	//	10" tablets
		{
			// width >= 768 and height == 1024 ( iPad )
			// or width between 1025 and 1280 and height is less than 1023
			// or the same with width and height interchanged

			// All iPads including mini go here, because there is no way to tell full-sized iPad from mini.
			return "@media (device-width: 768px) and (device-height: 1024px)".

			// The rest of contemporary 10" devices supposedly go here
				" , (min-device-width: 1025px) and (max-device-width: 1280px) and (max-device-height: 1023px) , (min-device-height: 1025px) and (max-device-height: 1280px) and (max-device-width: 1023px)";
		}
		else if( $device == TABLET_7_IN )
		{
			// width between 401 and 1024 and height between 401 and 800
			// or the same with width and height interchanged

			return "@media (min-device-height: 401px) and (max-device-height: 800px) and (min-device-width: 401px) and (max-device-width: 1024px) , (min-device-height: 401px) and (min-device-width: 401px) and (max-device-height: 1024px) and (max-device-width: 800px)";
		}

		else if( $device == SMARTPHONE_LANDSCAPE )
		{
			// landscape mode and width or height no more than 400
			return "@media (orientation: landscape) and (max-device-height: 400px), (orientation: landscape) and (max-device-width: 400px)";
		}

		else if( $device == SMARTPHONE_PORTRAIT )
		{
			// potrait mode and width or height no more than 400
			return "@media (orientation: portrait) and (max-device-height: 400px), (orientation: portrait) and (max-device-width: 400px)";
		}
	}

	/**
	 * @return Mixed
	 */
	public static function getForLogin()
	{
		return new ProjectSettings("payroll_system_user", PAGE_LIST);
	}

	/**
	 * Returns the list of the dashboard search fields.
	 * Each field is an array of "table", "field" pairs.
	 */
	function getDashboardSearchFields()
	{
		return $this->getTableData(".searchFields");
	}
	/**
	 * Returns the list of the dashboard elements
	 * Each element is represented by the array of "name", "table" and "type"
	 */
	function getDashboardElements()
	{
		return $this->getTableData(".dashElements");
	}

	/**
	 * @param String dashElementName
	 * @return Array
	 */
	function getDashboardElementData( $dashElementName )
	{
		$dElements = $this->getTableData(".dashElements");
		foreach( $dElements as $dElemData )
		{
			if( $dElemData["elementName"] == $dashElementName )
				return $dElemData;
		}
		return array();
	}

	/**
	 * @return Number
	 */
	function getAfterAddAction()
	{
		return $this->getTableData(".afterAddAction");
	}

	/**
	 * @return String
	 */
	function getAADetailTable()
	{
		return $this->getTableData(".afterAddActionDetTable");
	}

	/**
	 * @return Boolean
	 */
	function checkClosePopupAfterAdd()
	{
		return $this->getTableData(".closePopupAfterAdd");
	}

	/**
	 * @return Number
	 */
	function getAfterEditAction()
	{
		return $this->getTableData(".afterEditAction");
	}

	/**
	 * @return String
	 */
	function getAEDetailTable()
	{
		return $this->getTableData(".afterEditActionDetTable");
	}

	/**
	 * @return Boolean
	 */
	function checkClosePopupAfterEdit()
	{
		return $this->getTableData(".closePopupAfterEdit");
	}

	function getMapIcon($field, $data)
	{
		if( !$this->isMapIconCustom($field) ) {
			$mapData = $this->getMapData($field);
			if( $mapData["mapIcon"] != "" )
				return "images/menuicons/" . $mapData["mapIcon"];
			return "";
		}
		else
		{
			return getCustomMapIcon($field, "", $data);
		}
	}

	/**
	 * @param String dashElementName
	 * @param Array data
	 * @return String
	 */
	function getDashMapIcon( $dashElementName, $data )
	{
		$dashElementData = $this->getDashboardElementData( $dashElementName );

		if( $dashElementData["isMarkerIconCustom"] )
			return getDashMapCustomIcon( $this->_table, $dashElementName, $data );

		if( $dashElementData["iconF"] )
			return $dashElementData["iconF"];

		return "";
	}

	function isMapIconCustom($field)
	{
		$mapData = $this->getMapData($field);
		return $mapData["isMapIconCustom"];
	}

	function getDetailsBadgeColor()
	{
		return $this->getTableData(".badgeColor");
	}

	function & helperFormItems()
	{
		return $this->getPageOption("layoutHelper", "formItems");
	}
	function & helperItemsByType()
	{
		return $this->getPageOption("layoutHelper", "itemsByType");
	}

	function & allFieldItems()
	{
		return $this->getPageOption("fields", "fieldItems");
	}

	function & helperItemVisibility()
	{
		return $this->getPageOption("layoutHelper", "itemVisibility");
	}

	function & helperCellMaps()
	{
		return $this->getPageOption("layoutHelper", "cellMaps");
	}

	function detailsShowCount($dTable)
	{
		return $this->getPageOption("details", $dTable, "showCount" );
	}

	function detailsHideEmpty($dTable)
	{
		return $this->getPageOption("details", $dTable, "hideEmptyChild" );
	}
	
	function detailsHideEmptyPreview($dTable)
	{
		return $this->getPageOption("details", $dTable, "hideEmptyPreview" );
	}	

	function detailsPreview($dTable)
	{
		return $this->getPageOption("details", $dTable, "displayPreview" );
	}

	function detailsProceedLink($dTable)
	{
		return $this->getPageOption("details", $dTable, "showProceedLink" );
	}

	function detailsPrint($dTable)
	{
		return $this->getPageOption("details", $dTable, "printDetails" );
	}

	function detailsLinks()
	{
		return $this->getPageOption("allDetails", "linkType" );
	}
	function detailsPageId( $dTable )
	{
		return $this->getPageOption("details", $dTable, "previewPageId" );
	}

	function masterPreview( $mTable )
	{
		return $this->getPageOption("master", $mTable, "preview" );
	}

	/**
	 * @return boolean if success otherwise false
	 */
	function hasMap()
	{
		return !!$this->getPageOption("events", "maps" );
	}
	function maps()
	{
		return $this->getPageOption("events", "maps" );
	}

	function mapsData()
	{
		return $this->getPageOption("events", "mapsData" );
	}

	function buttons()
	{
		return $this->getPageOption("events", "buttons" );
	}

	function getPageType( $page = "" )
	{
		if( !$page )
			$page = $this->_page;
		return $this->_auxTableData[ ".pages" ][ $page ];
	}

	/**
	 * returns page type without applying page permissions
	 */
	function getOriginalPageType( $page = "" )
	{
		if( !$page )
			$page = $this->_page;
		return $this->_auxTableData[ ".originalPages" ][ $page ];
	}

	function welcomeItems()
	{
		return $this->getPageOption("welcome", "welcomeItems");
	}

	function getMultipleImgMode( $field )
	{
		return $this->getFieldData( $field, "multipleImgMode" );
	}

	function getMaxImages( $field )
	{
		return $this->getFieldData( $field, "maxImages" );
	}
	function isGalleryEnabled( $field )
	{
		return $this->getFieldData( $field, "showGallery" );
	}
	function getGalleryMode( $field )
	{
		return $this->getFieldData( $field, "galleryMode" );
	}
	function getCaptionMode( $field )
	{
		return $this->getFieldData( $field, "captionMode" );
	}
	function getCaptionField( $field )
	{
		return $this->getFieldData( $field, "captionField" );
	}

	/**
	 * @return boolean
	 */
	function getImageBorder( $field )
	{
		return $this->getFieldData( $field, "imageBorder" );
	}

	function getImageFullWidth( $field )
	{
		return $this->getFieldData( $field, "imageFullWidth" );
	}

	/**
	 * Page exists
	 * @return Boolean
	 */
	function pageTypeAvailable( $pageType ) {
		$pagesByType =& $this->_auxTableData[".pagesByType"];
		return !!$pagesByType[ $pageType ];
	}

	function updatePages() {
		
		if( $this->_auxTableData[".pagesUpdated"] ) {
			return;
		}
 
		//	don't filter pages available to all users
		if( $this->_pageType == PAGE_LOGIN 
			|| $this->_pageType == PAGE_REGISTER
			|| $this->_pageType == PAGE_REMIND
			|| $this->_pageType == PAGE_REMIND_SUCCESS
		) {
			return;
		}


		$this->_auxTableData[".pagesUpdated"] = true;

		$restrictedPages = Security::getRestrictedPages( $this->_auxTable );
		if( !$restrictedPages ) {
			return;
		}
		$pages =& $this->_auxTableData[".pages"];
		$pagesByType =& $this->_auxTableData[".pagesByType"];
		$newPages = array();
		$defaultPages =& $this->_auxTableData[".defaultPages"];
		foreach( $pages as $p => $type ) {
			if( !$restrictedPages[$p] ) {
				$newPages[ $p ] = $type;
			}
			else {
				$idx = array_search( $p, $pagesByType[$type] );
				unset( $pagesByType[$type][$idx] );
				if( $defaultPages[ $type ] == $p ) {
					$defaultPages[ $type ] = "";
					//	pick the first available page
					foreach( $pagesByType[$type] as $d ) {
						$defaultPages[ $type ] = $d;
						break;
					}
				}
			}
		}
		$this->_auxTableData[".pages"] = &$newPages;
	}
	
	/**
	 * Revert pages list to original.
	 * This is called from Security::setRestrictedPages/setAllowedPages
	 */
	function resetPages() {
		unset( $this->_auxTableData[".pagesUpdated"] );
		$this->_auxTableData[".defaultPages"] = $this->_auxTableData[".originalDefaultPages"];
		$this->_auxTableData[".pages"] = $this->_auxTableData[".originalPages"];
	}

	/**
	 * returns array of page ids ignoring user permissions
	 */
	function getOriginalPagesByType( $pageType) {
		return $this->_auxTableData[".originalPagesByType"][ $pageType ];
	}

	function &getDataSourceOps()
	{
		return $this->_tableData[".operations"];
	}

	/**
	 * @return Boolean
	 */
	function groupChart() {
		return $this->getTableData( ".groupChart" );
	}

	/**
	 * @return Integer
	 */
	function chartLabelInterval() {
		return $this->getTableData( ".chartLabelInterval" );
	}

	/**
	 * @return Array
	 */
	function chartSeries() {
		return $this->getTableData( ".chartSeries" );
	}

	/**
	 * @return String
	 */
	function chartLabelField() {
		return $this->getTableData( ".chartLabelField" );
	}

	function getViewPageType() {
		return $this->_viewPage;
	}

	function spreadsheetGrid() {
		return $this->getPageOption( "list", "spreadsheetMode");
	}

	/**
	 * Tell whether the edit format involves file uploading
	 * @return boolean
	 */
	public static function uploadEditType( $editFormat ) {
		return $editFormat == EDIT_FORMAT_DATABASE_FILE || $editFormat == EDIT_FORMAT_DATABASE_IMAGE || $editFormat == EDIT_FORMAT_FILE;
	}
	
	public function addNewRecordAutomatically() {
		return $this->getPageOption( "list", "addNewRecordAutomatically" );
	}

	public function reorderRows() {
		return $this->getPageOption( "list", "reorderRows" ) && $this->reorderRowsField() != '';
	}

	public function reorderRowsField() {
		return $this->getPageOption( "list", "reorderRowsField");
	}

	public function inlineAddBottom() {
		return !!( $this->getPageOption( "list", "addToBottom") || $this->spreadsheetGrid() && $this->addNewRecordAutomatically() );
	}

}


function fillProjectEntites()
{
	global $projectEntities, $projectEntitiesReverse, $tablesByGoodName, $tablesByUpperCase, $tablesByUpperGoodname;
	if( count( $projectEntities ) )
		return;
	$projectEntities[ "attendance_summary" ] = array( "url" => "attendance_summary", "type" => 0 );
	$projectEntitiesReverse[ "attendance_summary" ] = "attendance_summary";
	$projectEntities[ "employees" ] = array( "url" => "employees", "type" => 0 );
	$projectEntitiesReverse[ "employees" ] = "employees";
	$projectEntities[ "process_payroll_detail" ] = array( "url" => "process_payroll_detail", "type" => 0 );
	$projectEntitiesReverse[ "process_payroll_detail" ] = "process_payroll_detail";
	$projectEntities[ "executive_manangement" ] = array( "url" => "executive_manangement", "type" => 0 );
	$projectEntitiesReverse[ "executive_manangement" ] = "executive_manangement";
	$projectEntities[ "company_info" ] = array( "url" => "company_info", "type" => 0 );
	$projectEntitiesReverse[ "company_info" ] = "company_info";
	$projectEntities[ "payroll_item" ] = array( "url" => "payroll_item", "type" => 0 );
	$projectEntitiesReverse[ "payroll_item" ] = "payroll_item";
	$projectEntities[ "payroll_entry" ] = array( "url" => "payroll_entry", "type" => 0 );
	$projectEntitiesReverse[ "payroll_entry" ] = "payroll_entry";
	$projectEntities[ "payroll_entry_detail" ] = array( "url" => "payroll_entry_detail", "type" => 0 );
	$projectEntitiesReverse[ "payroll_entry_detail" ] = "payroll_entry_detail";
	$projectEntities[ "cost_center" ] = array( "url" => "cost_center", "type" => 0 );
	$projectEntitiesReverse[ "cost_center" ] = "cost_center";
	$projectEntities[ "process_payroll_for_checking" ] = array( "url" => "process_payroll_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "process_payroll_for_checking" ] = "process_payroll_for_checking";
	$projectEntities[ "process_payroll_for_approval" ] = array( "url" => "process_payroll_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "process_payroll_for_approval" ] = "process_payroll_for_approval";
	$projectEntities[ "process_payroll_detail_for_checking" ] = array( "url" => "process_payroll_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "process_payroll_detail_for_checking" ] = "process_payroll_detail_for_checking";
	$projectEntities[ "process_payroll_detail_for_approval" ] = array( "url" => "process_payroll_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "process_payroll_detail_for_approval" ] = "process_payroll_detail_for_approval";
	$projectEntities[ "payroll_entry_for_checking" ] = array( "url" => "payroll_entry_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "payroll_entry_for_checking" ] = "payroll_entry_for_checking";
	$projectEntities[ "payroll_entry_for_approval" ] = array( "url" => "payroll_entry_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "payroll_entry_for_approval" ] = "payroll_entry_for_approval";
	$projectEntities[ "payroll_entry_detail_for_checking" ] = array( "url" => "payroll_entry_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "payroll_entry_detail_for_checking" ] = "payroll_entry_detail_for_checking";
	$projectEntities[ "payroll_entry_detail_for_approval" ] = array( "url" => "payroll_entry_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "payroll_entry_detail_for_approval" ] = "payroll_entry_detail_for_approval";
	$projectEntities[ "bio_upload" ] = array( "url" => "bio_upload", "type" => 0 );
	$projectEntitiesReverse[ "bio_upload" ] = "bio_upload";
	$projectEntities[ "attendance" ] = array( "url" => "attendance", "type" => 0 );
	$projectEntitiesReverse[ "attendance" ] = "attendance";
	$projectEntities[ "attendance_type" ] = array( "url" => "attendance_type", "type" => 0 );
	$projectEntitiesReverse[ "attendance_type" ] = "attendance_type";
	$projectEntities[ "schedule" ] = array( "url" => "schedule", "type" => 0 );
	$projectEntitiesReverse[ "schedule" ] = "schedule";
	$projectEntities[ "department" ] = array( "url" => "department", "type" => 0 );
	$projectEntitiesReverse[ "department" ] = "department";
	$projectEntities[ "position" ] = array( "url" => "position", "type" => 0 );
	$projectEntitiesReverse[ "position" ] = "position";
	$projectEntities[ "recuring_benefits" ] = array( "url" => "recuring_benefits", "type" => 0 );
	$projectEntitiesReverse[ "recuring_benefits" ] = "recuring_benefits";
	$projectEntities[ "recuring_deductions" ] = array( "url" => "recuring_deductions", "type" => 0 );
	$projectEntitiesReverse[ "recuring_deductions" ] = "recuring_deductions";
	$projectEntities[ "business_partner" ] = array( "url" => "business_partner", "type" => 0 );
	$projectEntitiesReverse[ "business_partner" ] = "business_partner";
	$projectEntities[ "applicant_requirements" ] = array( "url" => "applicant_requirements", "type" => 0 );
	$projectEntitiesReverse[ "applicant_requirements" ] = "applicant_requirements";
	$projectEntities[ "employee_requirements" ] = array( "url" => "employee_requirements", "type" => 0 );
	$projectEntitiesReverse[ "employee_requirements" ] = "employee_requirements";
	$projectEntities[ "gov_or_payments" ] = array( "url" => "gov_or_payments", "type" => 0 );
	$projectEntitiesReverse[ "gov_or_payments" ] = "gov_or_payments";
	$projectEntities[ "gov_or_records" ] = array( "url" => "gov_or_records", "type" => 0 );
	$projectEntitiesReverse[ "gov_or_records" ] = "gov_or_records";
	$projectEntities[ "accident" ] = array( "url" => "accident", "type" => 0 );
	$projectEntitiesReverse[ "accident" ] = "accident";
	$projectEntities[ "employees_payroll_setup" ] = array( "url" => "employees_payroll_setup", "type" => 1 );
	$projectEntitiesReverse[ "employees_payroll_setup" ] = "employees_payroll_setup";
	$projectEntities[ "employee_dependents" ] = array( "url" => "employee_dependents", "type" => 0 );
	$projectEntitiesReverse[ "employee_dependents" ] = "employee_dependents";
	$projectEntities[ "over_time" ] = array( "url" => "over_time", "type" => 0 );
	$projectEntitiesReverse[ "over_time" ] = "over_time";
	$projectEntities[ "leave" ] = array( "url" => "leave", "type" => 0 );
	$projectEntitiesReverse[ "leave" ] = "leave";
	$projectEntities[ "undertime" ] = array( "url" => "undertime", "type" => 0 );
	$projectEntitiesReverse[ "undertime" ] = "undertime";
	$projectEntities[ "change_time" ] = array( "url" => "change_time", "type" => 0 );
	$projectEntitiesReverse[ "change_time" ] = "change_time";
	$projectEntities[ "fieldwork" ] = array( "url" => "fieldwork", "type" => 0 );
	$projectEntitiesReverse[ "fieldwork" ] = "fieldwork";
	$projectEntities[ "payroll_account_type" ] = array( "url" => "payroll_account_type", "type" => 0 );
	$projectEntitiesReverse[ "payroll_account_type" ] = "payroll_account_type";
	$projectEntities[ "bank" ] = array( "url" => "bank", "type" => 0 );
	$projectEntitiesReverse[ "bank" ] = "bank";
	$projectEntities[ "sss" ] = array( "url" => "sss", "type" => 0 );
	$projectEntitiesReverse[ "sss" ] = "sss";
	$projectEntities[ "pagibig" ] = array( "url" => "pagibig", "type" => 0 );
	$projectEntitiesReverse[ "pagibig" ] = "pagibig";
	$projectEntities[ "annual_tax_table" ] = array( "url" => "annual_tax_table", "type" => 0 );
	$projectEntitiesReverse[ "annual_tax_table" ] = "annual_tax_table";
	$projectEntities[ "holiday" ] = array( "url" => "holiday", "type" => 0 );
	$projectEntitiesReverse[ "holiday" ] = "holiday";
	$projectEntities[ "philhealth" ] = array( "url" => "philhealth", "type" => 0 );
	$projectEntitiesReverse[ "philhealth" ] = "philhealth";
	$projectEntities[ "payroll_report_center" ] = array( "url" => "payroll_report_center", "type" => 0 );
	$projectEntitiesReverse[ "payroll_report_center" ] = "payroll_report_center";
	$projectEntities[ "employment_history" ] = array( "url" => "employment_history", "type" => 0 );
	$projectEntitiesReverse[ "employment_history" ] = "employment_history";
	$projectEntities[ "tranasactions_for_approval" ] = array( "url" => "tranasactions_for_approval", "type" => 0 );
	$projectEntitiesReverse[ "tranasactions_for_approval" ] = "tranasactions_for_approval";
	$projectEntities[ "approver_list" ] = array( "url" => "approver_list", "type" => 0 );
	$projectEntitiesReverse[ "approver_list" ] = "approver_list";
	$projectEntities[ "Fieldwork_for_approval" ] = array( "url" => "Fieldwork_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Fieldwork_for_approval" ] = "Fieldwork_for_approval";
	$projectEntities[ "employee_project" ] = array( "url" => "employee_project", "type" => 0 );
	$projectEntitiesReverse[ "employee_project" ] = "employee_project";
	$projectEntities[ "employee_project_task" ] = array( "url" => "employee_project_task", "type" => 0 );
	$projectEntitiesReverse[ "employee_project_task" ] = "employee_project_task";
	$projectEntities[ "activity_log" ] = array( "url" => "activity_log", "type" => 0 );
	$projectEntitiesReverse[ "activity_log" ] = "activity_log";
	$projectEntities[ "payroll_system_user" ] = array( "url" => "payroll_system_user", "type" => 0 );
	$projectEntitiesReverse[ "payroll_system_user" ] = "payroll_system_user";
	$projectEntities[ "chart_of_accounts" ] = array( "url" => "chart_of_accounts", "type" => 0 );
	$projectEntitiesReverse[ "chart_of_accounts" ] = "chart_of_accounts";
	$projectEntities[ "request_for_payment" ] = array( "url" => "request_for_payment", "type" => 0 );
	$projectEntitiesReverse[ "request_for_payment" ] = "request_for_payment";
	$projectEntities[ "request_for_payment_detail" ] = array( "url" => "request_for_payment_detail", "type" => 0 );
	$projectEntitiesReverse[ "request_for_payment_detail" ] = "request_for_payment_detail";
	$projectEntities[ "recurring_benefits_detail" ] = array( "url" => "recurring_benefits_detail", "type" => 0 );
	$projectEntitiesReverse[ "recurring_benefits_detail" ] = "recurring_benefits_detail";
	$projectEntities[ "recurring_deductions_detail" ] = array( "url" => "recurring_deductions_detail", "type" => 0 );
	$projectEntitiesReverse[ "recurring_deductions_detail" ] = "recurring_deductions_detail";
	$projectEntities[ "Dashboard" ] = array( "url" => "Dashboard", "type" => 4 );
	$projectEntitiesReverse[ "Dashboard" ] = "Dashboard";
	$projectEntities[ "Training_Dashboard" ] = array( "url" => "Training_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Training_Dashboard" ] = "Training_Dashboard";
	$projectEntities[ "Seminar_Dashboard" ] = array( "url" => "Seminar_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Seminar_Dashboard" ] = "Seminar_Dashboard";
	$projectEntities[ "Accident_Dashboard" ] = array( "url" => "Accident_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Accident_Dashboard" ] = "Accident_Dashboard";
	$projectEntities[ "Employment_History_Dashboard" ] = array( "url" => "Employment_History_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Employment_History_Dashboard" ] = "Employment_History_Dashboard";
	$projectEntities[ "Overtime_for_checking" ] = array( "url" => "Overtime_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Overtime_for_checking" ] = "Overtime_for_checking";
	$projectEntities[ "Overtime_for_approval" ] = array( "url" => "Overtime_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Overtime_for_approval" ] = "Overtime_for_approval";
	$projectEntities[ "Undertime_for_approval" ] = array( "url" => "Undertime_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Undertime_for_approval" ] = "Undertime_for_approval";
	$projectEntities[ "Undertime_for_checking" ] = array( "url" => "Undertime_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Undertime_for_checking" ] = "Undertime_for_checking";
	$projectEntities[ "Change_Time_Schedule_for_approval" ] = array( "url" => "Change_Time_Schedule_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Change_Time_Schedule_for_approval" ] = "Change_Time_Schedule_for_approval";
	$projectEntities[ "Change_Time_Schedule_for_checking" ] = array( "url" => "Change_Time_Schedule_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Change_Time_Schedule_for_checking" ] = "Change_Time_Schedule_for_checking";
	$projectEntities[ "item" ] = array( "url" => "item", "type" => 0 );
	$projectEntitiesReverse[ "item" ] = "item";
	$projectEntities[ "uom" ] = array( "url" => "uom", "type" => 0 );
	$projectEntitiesReverse[ "uom" ] = "uom";
	$projectEntities[ "vat_type" ] = array( "url" => "vat_type", "type" => 0 );
	$projectEntitiesReverse[ "vat_type" ] = "vat_type";
	$projectEntities[ "vat_rate" ] = array( "url" => "vat_rate", "type" => 0 );
	$projectEntitiesReverse[ "vat_rate" ] = "vat_rate";
	$projectEntities[ "phase" ] = array( "url" => "phase", "type" => 0 );
	$projectEntitiesReverse[ "phase" ] = "phase";
	$projectEntities[ "Leave_for_checking" ] = array( "url" => "Leave_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Leave_for_checking" ] = "Leave_for_checking";
	$projectEntities[ "Leave_for_approval" ] = array( "url" => "Leave_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Leave_for_approval" ] = "Leave_for_approval";
	$projectEntities[ "currency" ] = array( "url" => "currency", "type" => 0 );
	$projectEntitiesReverse[ "currency" ] = "currency";
	$projectEntities[ "credit_terms" ] = array( "url" => "credit_terms", "type" => 0 );
	$projectEntitiesReverse[ "credit_terms" ] = "credit_terms";
	$projectEntities[ "discounts" ] = array( "url" => "discounts", "type" => 0 );
	$projectEntitiesReverse[ "discounts" ] = "discounts";
	$projectEntities[ "purchase_request" ] = array( "url" => "purchase_request", "type" => 0 );
	$projectEntitiesReverse[ "purchase_request" ] = "purchase_request";
	$projectEntities[ "purchase_request_detail" ] = array( "url" => "purchase_request_detail", "type" => 0 );
	$projectEntitiesReverse[ "purchase_request_detail" ] = "purchase_request_detail";
	$projectEntities[ "currency_exchange_rate" ] = array( "url" => "currency_exchange_rate", "type" => 0 );
	$projectEntitiesReverse[ "currency_exchange_rate" ] = "currency_exchange_rate";
	$projectEntities[ "Purchase_Request_for_approval" ] = array( "url" => "Purchase_Request_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "Purchase_Request_for_approval" ] = "Purchase_Request_for_approval";
	$projectEntities[ "Purchase_Request_for_checking" ] = array( "url" => "Purchase_Request_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Purchase_Request_for_checking" ] = "Purchase_Request_for_checking";
	$projectEntities[ "purchase_request_detail_for_approval" ] = array( "url" => "purchase_request_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "purchase_request_detail_for_approval" ] = "purchase_request_detail_for_approval";
	$projectEntities[ "purchase_request_detail_for_checking" ] = array( "url" => "purchase_request_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "purchase_request_detail_for_checking" ] = "purchase_request_detail_for_checking";
	$projectEntities[ "request_for_payment_for_approval" ] = array( "url" => "request_for_payment_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "request_for_payment_for_approval" ] = "request_for_payment_for_approval";
	$projectEntities[ "request_for_payment_for_checking" ] = array( "url" => "request_for_payment_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "request_for_payment_for_checking" ] = "request_for_payment_for_checking";
	$projectEntities[ "request_for_payment_detail_for_approval" ] = array( "url" => "request_for_payment_detail_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "request_for_payment_detail_for_approval" ] = "request_for_payment_detail_for_approval";
	$projectEntities[ "request_for_payment_detail_for_checking" ] = array( "url" => "request_for_payment_detail_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "request_for_payment_detail_for_checking" ] = "request_for_payment_detail_for_checking";
	$projectEntities[ "accounting_period" ] = array( "url" => "accounting_period", "type" => 0 );
	$projectEntitiesReverse[ "accounting_period" ] = "accounting_period";
	$projectEntities[ "fiscal_year" ] = array( "url" => "fiscal_year", "type" => 0 );
	$projectEntitiesReverse[ "fiscal_year" ] = "fiscal_year";
	$projectEntities[ "account_classification" ] = array( "url" => "account_classification", "type" => 0 );
	$projectEntitiesReverse[ "account_classification" ] = "account_classification";
	$projectEntities[ "financial_statement" ] = array( "url" => "financial_statement", "type" => 0 );
	$projectEntitiesReverse[ "financial_statement" ] = "financial_statement";
	$projectEntities[ "account_type" ] = array( "url" => "account_type", "type" => 0 );
	$projectEntitiesReverse[ "account_type" ] = "account_type";
	$projectEntities[ "account_general_classification" ] = array( "url" => "account_general_classification", "type" => 0 );
	$projectEntitiesReverse[ "account_general_classification" ] = "account_general_classification";
	$projectEntities[ "reimbursement" ] = array( "url" => "reimbursement", "type" => 0 );
	$projectEntitiesReverse[ "reimbursement" ] = "reimbursement";
	$projectEntities[ "reimbursement_detail" ] = array( "url" => "reimbursement_detail", "type" => 0 );
	$projectEntitiesReverse[ "reimbursement_detail" ] = "reimbursement_detail";
	$projectEntities[ "system_user" ] = array( "url" => "system_user", "type" => 0 );
	$projectEntitiesReverse[ "system_user" ] = "system_user";
	$projectEntities[ "serial_no" ] = array( "url" => "serial_no", "type" => 0 );
	$projectEntitiesReverse[ "serial_no" ] = "serial_no";
	$projectEntities[ "change_password" ] = array( "url" => "change_password", "type" => 1 );
	$projectEntitiesReverse[ "change_password" ] = "change_password";
	$projectEntities[ "aging_ap" ] = array( "url" => "aging_ap", "type" => 0 );
	$projectEntitiesReverse[ "aging_ap" ] = "aging_ap";
	$projectEntities[ "Birthday_Reminder" ] = array( "url" => "Birthday_Reminder", "type" => 2 );
	$projectEntitiesReverse[ "Birthday_Reminder" ] = "Birthday_Reminder";
	$projectEntities[ "check_formats" ] = array( "url" => "check_formats", "type" => 0 );
	$projectEntitiesReverse[ "check_formats" ] = "check_formats";
	$projectEntities[ "Missing_log" ] = array( "url" => "Missing_log", "type" => 2 );
	$projectEntitiesReverse[ "Missing_log" ] = "Missing_log";
	$projectEntities[ "upcomming_holiday" ] = array( "url" => "upcomming_holiday", "type" => 2 );
	$projectEntitiesReverse[ "upcomming_holiday" ] = "upcomming_holiday";
	$projectEntities[ "tax_status" ] = array( "url" => "tax_status", "type" => 0 );
	$projectEntitiesReverse[ "tax_status" ] = "tax_status";
	$projectEntities[ "Loan_types" ] = array( "url" => "Loan_types", "type" => 1 );
	$projectEntitiesReverse[ "Loan_types" ] = "Loan_types";
	$projectEntities[ "Other_Benefits" ] = array( "url" => "Other_Benefits", "type" => 1 );
	$projectEntitiesReverse[ "Other_Benefits" ] = "Other_Benefits";
	$projectEntities[ "Other_Deductions" ] = array( "url" => "Other_Deductions", "type" => 1 );
	$projectEntitiesReverse[ "Other_Deductions" ] = "Other_Deductions";
	$projectEntities[ "Leave_types" ] = array( "url" => "Leave_types", "type" => 1 );
	$projectEntitiesReverse[ "Leave_types" ] = "Leave_types";
	$projectEntities[ "employee_recurring_benefit_detail" ] = array( "url" => "employee_recurring_benefit_detail", "type" => 0 );
	$projectEntitiesReverse[ "employee_recurring_benefit_detail" ] = "employee_recurring_benefit_detail";
	$projectEntities[ "employee_recurring_deduction_detail" ] = array( "url" => "employee_recurring_deduction_detail", "type" => 0 );
	$projectEntitiesReverse[ "employee_recurring_deduction_detail" ] = "employee_recurring_deduction_detail";
	$projectEntities[ "section" ] = array( "url" => "section", "type" => 0 );
	$projectEntitiesReverse[ "section" ] = "section";
	$projectEntities[ "overtime_detail" ] = array( "url" => "overtime_detail", "type" => 1 );
	$projectEntitiesReverse[ "overtime_detail" ] = "overtime_detail";
	$projectEntities[ "leave_detail" ] = array( "url" => "leave_detail", "type" => 1 );
	$projectEntitiesReverse[ "leave_detail" ] = "leave_detail";
	$projectEntities[ "change_time_detail" ] = array( "url" => "change_time_detail", "type" => 1 );
	$projectEntitiesReverse[ "change_time_detail" ] = "change_time_detail";
	$projectEntities[ "fieldwork_detail" ] = array( "url" => "fieldwork_detail", "type" => 1 );
	$projectEntitiesReverse[ "fieldwork_detail" ] = "fieldwork_detail";
	$projectEntities[ "changetime_file" ] = array( "url" => "changetime_file", "type" => 0 );
	$projectEntitiesReverse[ "changetime_file" ] = "changetime_file";
	$projectEntities[ "changetime_file_for_approval" ] = array( "url" => "changetime_file_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "changetime_file_for_approval" ] = "changetime_file_for_approval";
	$projectEntities[ "changetime_file_for_checking" ] = array( "url" => "changetime_file_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "changetime_file_for_checking" ] = "changetime_file_for_checking";
	$projectEntities[ "fieldwork_file" ] = array( "url" => "fieldwork_file", "type" => 0 );
	$projectEntitiesReverse[ "fieldwork_file" ] = "fieldwork_file";
	$projectEntities[ "attendance_logs" ] = array( "url" => "attendance_logs", "type" => 2 );
	$projectEntitiesReverse[ "attendance_logs" ] = "attendance_logs";
	$projectEntities[ "Supplier" ] = array( "url" => "Supplier", "type" => 1 );
	$projectEntitiesReverse[ "Supplier" ] = "Supplier";
	$projectEntities[ "Fieldwork_for_checking" ] = array( "url" => "Fieldwork_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "Fieldwork_for_checking" ] = "Fieldwork_for_checking";
	$projectEntities[ "witholding" ] = array( "url" => "witholding", "type" => 0 );
	$projectEntitiesReverse[ "witholding" ] = "witholding";
	$projectEntities[ "payroll_terms" ] = array( "url" => "payroll_terms", "type" => 0 );
	$projectEntitiesReverse[ "payroll_terms" ] = "payroll_terms";
	$projectEntities[ "violation" ] = array( "url" => "violation", "type" => 0 );
	$projectEntitiesReverse[ "violation" ] = "violation";
	$projectEntities[ "violation_type" ] = array( "url" => "violation_type", "type" => 0 );
	$projectEntitiesReverse[ "violation_type" ] = "violation_type";
	$projectEntities[ "loan" ] = array( "url" => "loan", "type" => 0 );
	$projectEntitiesReverse[ "loan" ] = "loan";
	$projectEntities[ "loan__detail" ] = array( "url" => "loan__detail", "type" => 0 );
	$projectEntitiesReverse[ "loan__detail" ] = "loan__detail";
	$projectEntities[ "leave_credits" ] = array( "url" => "leave_credits", "type" => 0 );
	$projectEntitiesReverse[ "leave_credits" ] = "leave_credits";
	$projectEntities[ "For_Approval_Dashboard" ] = array( "url" => "For_Approval_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "For_Approval_Dashboard" ] = "For_Approval_Dashboard";
	$projectEntities[ "For_Checking_Dashboard" ] = array( "url" => "For_Checking_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "For_Checking_Dashboard" ] = "For_Checking_Dashboard";
	$projectEntities[ "payment_method" ] = array( "url" => "payment_method", "type" => 0 );
	$projectEntitiesReverse[ "payment_method" ] = "payment_method";
	$projectEntities[ "attendance_summary_dashboard_report" ] = array( "url" => "attendance_summary_dashboard_report", "type" => 2 );
	$projectEntitiesReverse[ "attendance_summary_dashboard_report" ] = "attendance_summary_dashboard_report";
	$projectEntities[ "attendance_dashboard_report" ] = array( "url" => "attendance_dashboard_report", "type" => 2 );
	$projectEntitiesReverse[ "attendance_dashboard_report" ] = "attendance_dashboard_report";
	$projectEntities[ "post_attendance" ] = array( "url" => "post_attendance", "type" => 0 );
	$projectEntitiesReverse[ "post_attendance" ] = "post_attendance";
	$projectEntities[ "announcement" ] = array( "url" => "announcement", "type" => 0 );
	$projectEntitiesReverse[ "announcement" ] = "announcement";
	$projectEntities[ "timelog_for_checking" ] = array( "url" => "timelog_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "timelog_for_checking" ] = "timelog_for_checking";
	$projectEntities[ "timelog_for_approval" ] = array( "url" => "timelog_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "timelog_for_approval" ] = "timelog_for_approval";
	$projectEntities[ "medical" ] = array( "url" => "medical", "type" => 0 );
	$projectEntitiesReverse[ "medical" ] = "medical";
	$projectEntities[ "company" ] = array( "url" => "company", "type" => 0 );
	$projectEntitiesReverse[ "company" ] = "company";
	$projectEntities[ "Violation_Dashboard" ] = array( "url" => "Violation_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Violation_Dashboard" ] = "Violation_Dashboard";
	$projectEntities[ "certifications" ] = array( "url" => "certifications", "type" => 0 );
	$projectEntitiesReverse[ "certifications" ] = "certifications";
	$projectEntities[ "award" ] = array( "url" => "award", "type" => 0 );
	$projectEntitiesReverse[ "award" ] = "award";
	$projectEntities[ "examination" ] = array( "url" => "examination", "type" => 0 );
	$projectEntitiesReverse[ "examination" ] = "examination";
	$projectEntities[ "Medical_Dashboard" ] = array( "url" => "Medical_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Medical_Dashboard" ] = "Medical_Dashboard";
	$projectEntities[ "Certification_Dashboard" ] = array( "url" => "Certification_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Certification_Dashboard" ] = "Certification_Dashboard";
	$projectEntities[ "Award_Dashboard" ] = array( "url" => "Award_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Award_Dashboard" ] = "Award_Dashboard";
	$projectEntities[ "Examination_Dashboard" ] = array( "url" => "Examination_Dashboard", "type" => 2 );
	$projectEntitiesReverse[ "Examination_Dashboard" ] = "Examination_Dashboard";
	$projectEntities[ "basic_system_audit" ] = array( "url" => "basic_system_audit", "type" => 0 );
	$projectEntitiesReverse[ "basic_system_audit" ] = "basic_system_audit";
	$projectEntities[ "training" ] = array( "url" => "training", "type" => 0 );
	$projectEntitiesReverse[ "training" ] = "training";
	$projectEntities[ "employee_application" ] = array( "url" => "employee_application", "type" => 0 );
	$projectEntitiesReverse[ "employee_application" ] = "employee_application";
	$projectEntities[ "organizations" ] = array( "url" => "organizations", "type" => 0 );
	$projectEntitiesReverse[ "organizations" ] = "organizations";
	$projectEntities[ "location" ] = array( "url" => "location", "type" => 0 );
	$projectEntitiesReverse[ "location" ] = "location";
	$projectEntities[ "cost_center2" ] = array( "url" => "cost_center2", "type" => 0 );
	$projectEntitiesReverse[ "cost_center2" ] = "cost_center2";
	$projectEntities[ "Expiring Employee Recurring Benefits Report" ] = array( "url" => "Expiring_Employee_Recurring_Benefits_Report", "type" => 2 );
	$projectEntitiesReverse[ "Expiring_Employee_Recurring_Benefits_Report" ] = "Expiring Employee Recurring Benefits Report";
	$projectEntities[ "training_details" ] = array( "url" => "training_details", "type" => 0 );
	$projectEntitiesReverse[ "training_details" ] = "training_details";
	$projectEntities[ "admin_rights" ] = array( "url" => "admin_rights", "type" => 1 );
	$projectEntitiesReverse[ "admin_rights" ] = "admin_rights";
	$projectEntities[ "admin_members" ] = array( "url" => "admin_members", "type" => 1 );
	$projectEntitiesReverse[ "admin_members" ] = "admin_members";
	$projectEntities[ "admin_users" ] = array( "url" => "admin_users", "type" => 1 );
	$projectEntitiesReverse[ "admin_users" ] = "admin_users";
	$projectEntities[ "Loan_for_approval" ] = array( "url" => "loan_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "loan_for_approval" ] = "Loan_for_approval";
	$projectEntities[ "Loan_for_checking" ] = array( "url" => "loan_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "loan_for_checking" ] = "Loan_for_checking";
	$projectEntities[ "Loanx" ] = array( "url" => "loanx", "type" => 1 );
	$projectEntitiesReverse[ "loanx" ] = "Loanx";
	$projectEntities[ "Loanx2" ] = array( "url" => "loanx2", "type" => 1 );
	$projectEntitiesReverse[ "loanx2" ] = "Loanx2";
	$projectEntities[ "schedule_import" ] = array( "url" => "schedule_import", "type" => 1 );
	$projectEntitiesReverse[ "schedule_import" ] = "schedule_import";
	$projectEntities[ "process" ] = array( "url" => "process", "type" => 1 );
	$projectEntitiesReverse[ "process" ] = "process";
	$projectEntities[ "process_payroll" ] = array( "url" => "process_payroll", "type" => 0 );
	$projectEntitiesReverse[ "process_payroll" ] = "process_payroll";
	$projectEntities[ "additional_holiday_entry" ] = array( "url" => "additional_holiday_entry", "type" => 0 );
	$projectEntitiesReverse[ "additional_holiday_entry" ] = "additional_holiday_entry";
	$projectEntities[ "HR_Reports" ] = array( "url" => "hr_reports", "type" => 1 );
	$projectEntitiesReverse[ "hr_reports" ] = "HR_Reports";
	$projectEntities[ "weekly_schedule" ] = array( "url" => "weekly_schedule", "type" => 0 );
	$projectEntitiesReverse[ "weekly_schedule" ] = "weekly_schedule";
	$projectEntities[ "IT_report_center" ] = array( "url" => "it_report_center", "type" => 1 );
	$projectEntitiesReverse[ "it_report_center" ] = "IT_report_center";
	$projectEntities[ "sr_ops_report" ] = array( "url" => "sr_ops_report", "type" => 1 );
	$projectEntitiesReverse[ "sr_ops_report" ] = "sr_ops_report";
	$projectEntities[ "process_payroll_finalpay" ] = array( "url" => "process_payroll_finalpay", "type" => 0 );
	$projectEntitiesReverse[ "process_payroll_finalpay" ] = "process_payroll_finalpay";
	$projectEntities[ "seminar" ] = array( "url" => "seminar", "type" => 0 );
	$projectEntitiesReverse[ "seminar" ] = "seminar";
	$projectEntities[ "seminar_details" ] = array( "url" => "seminar_details", "type" => 0 );
	$projectEntitiesReverse[ "seminar_details" ] = "seminar_details";
	$projectEntities[ "Reimbursement_for_checking" ] = array( "url" => "reimbursement_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "reimbursement_for_checking" ] = "Reimbursement_for_checking";
	$projectEntities[ "Reimbursement_for_approval" ] = array( "url" => "reimbursement_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "reimbursement_for_approval" ] = "Reimbursement_for_approval";
	$projectEntities[ "holiday_overtime" ] = array( "url" => "holiday_overtime", "type" => 1 );
	$projectEntitiesReverse[ "holiday_overtime" ] = "holiday_overtime";
	$projectEntities[ "holiday_overtime_for_approval" ] = array( "url" => "holiday_overtime_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "holiday_overtime_for_approval" ] = "holiday_overtime_for_approval";
	$projectEntities[ "holiday_overtime_for_checking" ] = array( "url" => "holiday_overtime_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "holiday_overtime_for_checking" ] = "holiday_overtime_for_checking";
	$projectEntities[ "missing_logs" ] = array( "url" => "missing_logs", "type" => 0 );
	$projectEntitiesReverse[ "missing_logs" ] = "missing_logs";
	$projectEntities[ "Missing_Logs_for_checking" ] = array( "url" => "missing_logs_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "missing_logs_for_checking" ] = "Missing_Logs_for_checking";
	$projectEntities[ "Missing_Logs_for_approval" ] = array( "url" => "missing_logs_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "missing_logs_for_approval" ] = "Missing_Logs_for_approval";
	$projectEntities[ "flexben" ] = array( "url" => "flexben", "type" => 0 );
	$projectEntitiesReverse[ "flexben" ] = "flexben";
	$projectEntities[ "cut_off" ] = array( "url" => "cut_off", "type" => 0 );
	$projectEntitiesReverse[ "cut_off" ] = "cut_off";
	$projectEntities[ "flexben_type" ] = array( "url" => "flexben_type", "type" => 0 );
	$projectEntitiesReverse[ "flexben_type" ] = "flexben_type";
	$projectEntities[ "Flexben_for_checking" ] = array( "url" => "flexben_for_checking", "type" => 1 );
	$projectEntitiesReverse[ "flexben_for_checking" ] = "Flexben_for_checking";
	$projectEntities[ "Flexben_for_approval" ] = array( "url" => "flexben_for_approval", "type" => 1 );
	$projectEntitiesReverse[ "flexben_for_approval" ] = "Flexben_for_approval";
	$projectEntities[ "payroll_ticketing" ] = array( "url" => "payroll_ticketing", "type" => 0 );
	$projectEntitiesReverse[ "payroll_ticketing" ] = "payroll_ticketing";
	$projectEntities[ "payroll_ticketing(Open)" ] = array( "url" => "payroll_ticketing_open_", "type" => 1 );
	$projectEntitiesReverse[ "payroll_ticketing_open_" ] = "payroll_ticketing(Open)";
	$projectEntities[ "job_grade" ] = array( "url" => "job_grade", "type" => 0 );
	$projectEntitiesReverse[ "job_grade" ] = "job_grade";
	$projectEntities[ "level" ] = array( "url" => "level", "type" => 0 );
	$projectEntitiesReverse[ "level" ] = "level";
	$projectEntities[ "Payroll Generation Per Department Pie" ] = array( "url" => "payroll_generation_per_department_pie", "type" => 3 );
	$projectEntitiesReverse[ "payroll_generation_per_department_pie" ] = "Payroll Generation Per Department Pie";
	$projectEntities[ "Payroll Generation" ] = array( "url" => "payroll_generation", "type" => 3 );
	$projectEntitiesReverse[ "payroll_generation" ] = "Payroll Generation";
	$projectEntities[ "Overtime_approver1" ] = array( "url" => "overtime_approver1", "type" => 1 );
	$projectEntitiesReverse[ "overtime_approver1" ] = "Overtime_approver1";
	$projectEntities[ "Approver1_Dashboard" ] = array( "url" => "approver1_dashboard", "type" => 2 );
	$projectEntitiesReverse[ "approver1_dashboard" ] = "Approver1_Dashboard";
	$projectEntities[ "Overtime_approver2" ] = array( "url" => "overtime_approver2", "type" => 1 );
	$projectEntitiesReverse[ "overtime_approver2" ] = "Overtime_approver2";
	$projectEntities[ "Overtime_approver3" ] = array( "url" => "overtime_approver3", "type" => 1 );
	$projectEntitiesReverse[ "overtime_approver3" ] = "Overtime_approver3";
	$projectEntities[ "Overtime_approver4" ] = array( "url" => "overtime_approver4", "type" => 1 );
	$projectEntitiesReverse[ "overtime_approver4" ] = "Overtime_approver4";
	$projectEntities[ "Approver2_Dashboard" ] = array( "url" => "approver2_dashboard", "type" => 2 );
	$projectEntitiesReverse[ "approver2_dashboard" ] = "Approver2_Dashboard";
	$projectEntities[ "Approver3_Dashboard" ] = array( "url" => "approver3_dashboard", "type" => 2 );
	$projectEntitiesReverse[ "approver3_dashboard" ] = "Approver3_Dashboard";
	$projectEntities[ "Approver4_Dashboard" ] = array( "url" => "approver4_dashboard", "type" => 2 );
	$projectEntitiesReverse[ "approver4_dashboard" ] = "Approver4_Dashboard";
	$projectEntities[ "Leave_approver1" ] = array( "url" => "leave_approver1", "type" => 1 );
	$projectEntitiesReverse[ "leave_approver1" ] = "Leave_approver1";
	$projectEntities[ "Leave_approver2" ] = array( "url" => "leave_approver2", "type" => 1 );
	$projectEntitiesReverse[ "leave_approver2" ] = "Leave_approver2";
	$projectEntities[ "Leave_approver3" ] = array( "url" => "leave_approver3", "type" => 1 );
	$projectEntitiesReverse[ "leave_approver3" ] = "Leave_approver3";
	$projectEntities[ "Leave_approver4" ] = array( "url" => "leave_approver4", "type" => 1 );
	$projectEntitiesReverse[ "leave_approver4" ] = "Leave_approver4";
	$projectEntities[ "Change_Time_Schedule_approver1" ] = array( "url" => "change_time_schedule_approver1", "type" => 1 );
	$projectEntitiesReverse[ "change_time_schedule_approver1" ] = "Change_Time_Schedule_approver1";
	$projectEntities[ "Change_Time_Schedule_approver2" ] = array( "url" => "change_time_schedule_approver2", "type" => 1 );
	$projectEntitiesReverse[ "change_time_schedule_approver2" ] = "Change_Time_Schedule_approver2";
	$projectEntities[ "Change_Time_Schedule_approver3" ] = array( "url" => "change_time_schedule_approver3", "type" => 1 );
	$projectEntitiesReverse[ "change_time_schedule_approver3" ] = "Change_Time_Schedule_approver3";
	$projectEntities[ "Change_Time_Schedule_approver4" ] = array( "url" => "change_time_schedule_approver4", "type" => 1 );
	$projectEntitiesReverse[ "change_time_schedule_approver4" ] = "Change_Time_Schedule_approver4";
	$projectEntities[ "Official_Business_approver1" ] = array( "url" => "official_business_approver1", "type" => 1 );
	$projectEntitiesReverse[ "official_business_approver1" ] = "Official_Business_approver1";
	$projectEntities[ "Official_Business_approver2" ] = array( "url" => "official_business_approver2", "type" => 1 );
	$projectEntitiesReverse[ "official_business_approver2" ] = "Official_Business_approver2";
	$projectEntities[ "Official_Business_approver3" ] = array( "url" => "official_business_approver3", "type" => 1 );
	$projectEntitiesReverse[ "official_business_approver3" ] = "Official_Business_approver3";
	$projectEntities[ "Official_Business_approver4" ] = array( "url" => "official_business_approver4", "type" => 1 );
	$projectEntitiesReverse[ "official_business_approver4" ] = "Official_Business_approver4";
	$projectEntities[ "Missing_Logs_approver1" ] = array( "url" => "missing_logs_approver1", "type" => 1 );
	$projectEntitiesReverse[ "missing_logs_approver1" ] = "Missing_Logs_approver1";
	$projectEntities[ "Missing_Logs_approver2" ] = array( "url" => "missing_logs_approver2", "type" => 1 );
	$projectEntitiesReverse[ "missing_logs_approver2" ] = "Missing_Logs_approver2";
	$projectEntities[ "Missing_Logs_approver3" ] = array( "url" => "missing_logs_approver3", "type" => 1 );
	$projectEntitiesReverse[ "missing_logs_approver3" ] = "Missing_Logs_approver3";
	$projectEntities[ "Missing_Logs_approver4" ] = array( "url" => "missing_logs_approver4", "type" => 1 );
	$projectEntitiesReverse[ "missing_logs_approver4" ] = "Missing_Logs_approver4";
	$projectEntities[ "Holiday_Overtime_approver1" ] = array( "url" => "holiday_overtime_approver1", "type" => 1 );
	$projectEntitiesReverse[ "holiday_overtime_approver1" ] = "Holiday_Overtime_approver1";
	$projectEntities[ "Holiday_Overtime_approver2" ] = array( "url" => "holiday_overtime_approver2", "type" => 1 );
	$projectEntitiesReverse[ "holiday_overtime_approver2" ] = "Holiday_Overtime_approver2";
	$projectEntities[ "Holiday_Overtime_approver3" ] = array( "url" => "holiday_overtime_approver3", "type" => 1 );
	$projectEntitiesReverse[ "holiday_overtime_approver3" ] = "Holiday_Overtime_approver3";
	$projectEntities[ "Holiday_Overtime_approver4" ] = array( "url" => "holiday_overtime_approver4", "type" => 1 );
	$projectEntitiesReverse[ "holiday_overtime_approver4" ] = "Holiday_Overtime_approver4";
	$projectEntities[ "year_end_pto_conversion" ] = array( "url" => "year_end_pto_conversion", "type" => 0 );
	$projectEntitiesReverse[ "year_end_pto_conversion" ] = "year_end_pto_conversion";
	$projectEntities[ "cto" ] = array( "url" => "cto", "type" => 0 );
	$projectEntitiesReverse[ "cto" ] = "cto";
	$projectEntities[ "pto_allowed_balance" ] = array( "url" => "pto_allowed_balance", "type" => 0 );
	$projectEntitiesReverse[ "pto_allowed_balance" ] = "pto_allowed_balance";
	$projectEntities[ "cto_approver1" ] = array( "url" => "cto_approver1", "type" => 1 );
	$projectEntitiesReverse[ "cto_approver1" ] = "cto_approver1";
	$projectEntities[ "cto_approver2" ] = array( "url" => "cto_approver2", "type" => 1 );
	$projectEntitiesReverse[ "cto_approver2" ] = "cto_approver2";
	$projectEntities[ "cto_approver3" ] = array( "url" => "cto_approver3", "type" => 1 );
	$projectEntitiesReverse[ "cto_approver3" ] = "cto_approver3";
	$projectEntities[ "cto_approver4" ] = array( "url" => "cto_approver4", "type" => 1 );
	$projectEntitiesReverse[ "cto_approver4" ] = "cto_approver4";
	$projectEntities[ "Change_Time_Schedule_approval" ] = array( "url" => "change_time_schedule_approval", "type" => 1 );
	$projectEntitiesReverse[ "change_time_schedule_approval" ] = "Change_Time_Schedule_approval";
	$projectEntities[ "Overtime_approval" ] = array( "url" => "overtime_approval", "type" => 1 );
	$projectEntitiesReverse[ "overtime_approval" ] = "Overtime_approval";
	$projectEntities[ "Leave_approval" ] = array( "url" => "leave_approval", "type" => 1 );
	$projectEntitiesReverse[ "leave_approval" ] = "Leave_approval";
	$projectEntities[ "Official_Business_approval" ] = array( "url" => "official_business_approval", "type" => 1 );
	$projectEntitiesReverse[ "official_business_approval" ] = "Official_Business_approval";
	$projectEntities[ "Missing_Logs_approval" ] = array( "url" => "missing_logs_approval", "type" => 1 );
	$projectEntitiesReverse[ "missing_logs_approval" ] = "Missing_Logs_approval";
	$projectEntities[ "Holiday_Overtime_approval" ] = array( "url" => "holiday_overtime_approval", "type" => 1 );
	$projectEntitiesReverse[ "holiday_overtime_approval" ] = "Holiday_Overtime_approval";
	$projectEntities[ "cto_approval" ] = array( "url" => "cto_approval", "type" => 1 );
	$projectEntitiesReverse[ "cto_approval" ] = "cto_approval";
	$projectEntities[ "Flexben_approval" ] = array( "url" => "flexben_approval", "type" => 1 );
	$projectEntitiesReverse[ "flexben_approval" ] = "Flexben_approval";
	$projectEntities[ "cancelled_process_payroll" ] = array( "url" => "cancelled_process_payroll", "type" => 1 );
	$projectEntitiesReverse[ "cancelled_process_payroll" ] = "cancelled_process_payroll";
	$projectEntities[ "Flexben_HR_approval" ] = array( "url" => "flexben_hr_approval", "type" => 1 );
	$projectEntitiesReverse[ "flexben_hr_approval" ] = "Flexben_HR_approval";
}

function findTable( $table ) {
	global $projectEntities, $projectEntitiesReverse;
	fillProjectEntites();

	if( isset( $projectEntities[ $table ] ) )
		return $table;
	$uTable = strtoupper( $table );

	foreach( $projectEntities as $t => $d ) {
		if( $uTable == strtoupper($t) )
			return $t;
		$gt = GoodFieldName($t);
		if( $table == $gt || $uTable == strtoupper( $gt ) )
			return $t;
	}

	//	check shortTableName
	if( isset( $projectEntitiesReverse[ $table ] ) )
		return $projectEntitiesReverse[ $table ];
	foreach( $projectEntitiesReverse as $t => $v ) {
		if( $uTable == strtoupper($t) )
			return $v;
	}
	return "";
}
//	return table short name
function GetTableURL( $table )
{
	if( $table == "<global>")
		return GLOBAL_PAGES_SHORT;
	global $strTableName, $projectEntities;

	if(!$table) {
		return "";
	}
	fillProjectEntites();

	if( isset( $projectEntities[ $table ] ) )
		return $projectEntities[ $table ][ "url" ];

	return "";
}

function GetEntityType($table = "")
{
	global $strTableName, $projectEntities;
	if(!$table)
		$table=$strTableName;
	fillProjectEntites();

	if( isset( $projectEntities[ $table ] ) )
		return $projectEntities[ $table ][ "type" ];

	return "";
}


//	return strTableName by short table name
function GetTableByShort( $shortTName )
{
	global $projectEntitiesReverse;
	fillProjectEntites();
	if(!$shortTName)
		return false;
	return $projectEntitiesReverse[ $shortTName ];
}

//	A
	$g_defaultOptionValues["Absolute"] = false;
	$g_defaultOptionValues["acceptFileTypes"] = ".+$";
	$g_defaultOptionValues["addFields"] = array();
	$g_defaultOptionValues["addPageEvents"] = false;
	$g_defaultOptionValues["ajaxCodeSnippetAdded"] = false;
	$g_defaultOptionValues["advSearchFields"] = array();
	$g_defaultOptionValues["afterAddAction"] = 1;
	$g_defaultOptionValues["afterAddActionDetTable"] = "";
	$g_defaultOptionValues["afterEditAction"] = 1;
	$g_defaultOptionValues["afterEditActionDetTable"] = "";
	$g_defaultOptionValues["AllowToAdd"] = false;
	$g_defaultOptionValues["allowFieldsReordering"] = false;
	$g_defaultOptionValues["allowShowHideFields"] = false;
	$g_defaultOptionValues["allSearchFields"] = array();
	$g_defaultOptionValues["arrGroupsPerPage"] = array();
	$g_defaultOptionValues["arrGridTabs"] = array();
	$g_defaultOptionValues["arrRecsPerPage"] = array();

	$g_defaultOptionValues["audioTitleField"] = "";
	$g_defaultOptionValues["audit"] = false;
	$g_defaultOptionValues["autoCompleteFields"] = array();
	$g_defaultOptionValues["autoCompleteFieldsAdd"] = array();
	$g_defaultOptionValues["autoCompleteFieldsEdit"] = array();
	$g_defaultOptionValues["autoCompleteFieldsOnEdit"] = false;
	$g_defaultOptionValues["AutoInc"] = false;
	$g_defaultOptionValues["autoUpload"] = false;
	$g_defaultOptionValues["autoUpdatable"] = false;
//	B
	$g_defaultOptionValues["badgeColor"] = "";
	$g_defaultOptionValues["bInlineAdd"] = false;
	$g_defaultOptionValues["bInlineEdit"] = false;
	$g_defaultOptionValues["bUpdateSelected"] = false;
	$g_defaultOptionValues["bIsEncrypted"] = false;
	$g_defaultOptionValues["bListPage"] = false;
	$g_defaultOptionValues["bPrinterPage"] = false;
	$g_defaultOptionValues["buttonsAdded"] = false;
//	C
	$g_defaultOptionValues["CompatibilityMode"] = false;
	$g_defaultOptionValues["categoryFields"] = array();
	$g_defaultOptionValues["ChartRefreshTime"] = 60;
	$g_defaultOptionValues["chartXml"] = "";
	$g_defaultOptionValues["chartType"] = "";
	$g_defaultOptionValues["closePopupAfterAdd"] = false;
	$g_defaultOptionValues["closePopupAfterEdit"] = false;
	$g_defaultOptionValues["controlWidth"] = 160;
	$g_defaultOptionValues["copy"] = false;
	$g_defaultOptionValues["CreateThumbnail"] = false;
	$g_defaultOptionValues["crossTabReport"] = false;
	$g_defaultOptionValues["CustomDisplay"] = false;
//	D
	$g_defaultOptionValues["detailsLinksOnList"] = DL_SINGLE;
	$g_defaultOptionValues["dashElements"] = array();
	$g_defaultOptionValues["dashCells"] = array();
	$g_defaultOptionValues["DateEditType"] = 0;
	$g_defaultOptionValues["DecimalDigits"] = "";
	$g_defaultOptionValues["defaultSearchOption"] = "";
	$g_defaultOptionValues["DefaultValue"] = "";
	$g_defaultOptionValues["delete"] = false;
	$g_defaultOptionValues["DeleteAssociatedFile"] = false;
	$g_defaultOptionValues["denyDuplicates"] = false;
	$g_defaultOptionValues["DependentLookups"] = array();
	$g_defaultOptionValues["descendingOrder"] = false;
	$g_defaultOptionValues["DisplayField"] = "";
//	E
	$g_defaultOptionValues["edit"] = false;
	$g_defaultOptionValues["editFields"] = array();
	$g_defaultOptionValues["EditFormat"] = "";
	$g_defaultOptionValues["EditParams"] = "";
	$g_defaultOptionValues["exportTo"] = false;
	$g_defaultOptionValues["exportFormatting"] = EXPORT_BOTH;
	$g_defaultOptionValues["exportDelimiter"] = ",";
//	F
	$g_defaultOptionValues["fieldIsVideoUrl"] = false;
	$g_defaultOptionValues["fieldIsImageUrl"] = false;
	$g_defaultOptionValues["FieldType"] = "";
	$g_defaultOptionValues["FieldPermissions"] = false;
	$g_defaultOptionValues["Filename"] = "";
	$g_defaultOptionValues["filterBy"] = 0;
	$g_defaultOptionValues["filterFields"] = array();
	$g_defaultOptionValues["filterFormat"] = FF_VALUE_LIST;
	$g_defaultOptionValues["filterIntervals"] = array();
	$g_defaultOptionValues["filterTotals"] = FT_NONE;
	$g_defaultOptionValues["filterMultiSelect"] = FM_NONE;
	$g_defaultOptionValues["filterCheckedMessageText"] = "";
	$g_defaultOptionValues["filterCheckedMessageType"] = "";
	$g_defaultOptionValues["filterUncheckedMessageText"] = "";
	$g_defaultOptionValues["filterUncheckedMessageType"] = "";
	$g_defaultOptionValues["filterSliderStepType"] = 1;
	$g_defaultOptionValues["filterSliderStepValue"] = 1;
	$g_defaultOptionValues["filterKnobsType"] = 0;
	$g_defaultOptionValues["filterApplyBtn"] = false;
	$g_defaultOptionValues["flexibleSearch"] = false;
	$g_defaultOptionValues["FormatTimeAttrs"] = array();
	$g_defaultOptionValues["freeInput"] = false;
	$g_defaultOptionValues["FullName"] = "";
	$g_defaultOptionValues["fieldEvents"] = false;
	$g_defaultOptionValues["fieldViewEvents"] = false;
//	G
	$g_defaultOptionValues["googleLikeFields"] = array();
	$g_defaultOptionValues["GoodName"] = "";
//	H
	$g_defaultOptionValues["hasEncryptedFields"] = false;
	$g_defaultOptionValues["hideEmptyFieldsOnView"] = false;
	$g_defaultOptionValues["HorizontalLookup"] = false;
	$g_defaultOptionValues["hlNewWindow"] = false;
	$g_defaultOptionValues["hlType"] = 0;
	$g_defaultOptionValues["hlLinkWordNameType"] = "";
	$g_defaultOptionValues["hlLinkWordText"] = "";
	$g_defaultOptionValues["hlTitleField"] = "";
	$g_defaultOptionValues["highlightSearchResults"] = false;
	$g_defaultOptionValues["HTML5InuptType"] = "text";
//	I
	$g_defaultOptionValues["Index"] = 0;
	$g_defaultOptionValues["InitialYearFactor"] = 100;
	$g_defaultOptionValues["ImageHeight"] = 0;
	$g_defaultOptionValues["ImageWidth"] = 0;
	//$g_defaultOptionValues["import"] = false;
	$g_defaultOptionValues["inlineAdd"] = false;
	$g_defaultOptionValues["inlineAddFields"] = array();
	$g_defaultOptionValues["inlineEdit"] = false;
	$g_defaultOptionValues["updateSelected"] = false;
	$g_defaultOptionValues["inlineEditFields"] = array();
	$g_defaultOptionValues["isDisplayLoading"] = false;
	$g_defaultOptionValues["tableType"] = "";
	$g_defaultOptionValues["IsRequired"] = false;
	$g_defaultOptionValues["insertNull"] = false;
	$g_defaultOptionValues["isResizeColumns"] = false;
	$g_defaultOptionValues["isSeparate"] = false;
	$g_defaultOptionValues["UpdateLatLng"] = false;
	$g_defaultOptionValues["isUseAjaxSuggest"] = false;
	$g_defaultOptionValues["isUseAudio"] = false;
	$g_defaultOptionValues["isUseFieldsMaps"] = false;
	$g_defaultOptionValues["isUseTimeForSearch"] = false;
	$g_defaultOptionValues["isUseToolTips"] = false;
	$g_defaultOptionValues["isUseVideo"] = false;
	$g_defaultOptionValues["listGridLayout"] = 0;
	$g_defaultOptionValues["isExistTotalFields"] = false;
	$g_defaultOptionValues["isViewPagePDFFitToPage"] = 1;
	$g_defaultOptionValues["isPrinterPagePDFFitToPage"] = 1;
	$g_defaultOptionValues["isLandscapeViewPDFOrientation"] = 0;
	$g_defaultOptionValues["isLandscapePrinterPagePDFOrientation"] = 0;
	$g_defaultOptionValues["isPrinterPagePDF"] = false;
	$g_defaultOptionValues["isViewPagePDF"] = false;
	$g_defaultOptionValues["isSQLExpression"] = false;
	$g_defaultOptionValues["isPrinterPageFitToPage"] = 1;
//	J
//	K
	$g_defaultOptionValues["Keys"] = array();
//	L
	$g_defaultOptionValues["Label"] = "";
	$g_defaultOptionValues["LookupConnId"] = "";
	$g_defaultOptionValues["LastYearFactor"] = 10;
	$g_defaultOptionValues["LCType"] = LCT_DROPDOWN;
	$g_defaultOptionValues["listPageId"] = "";
	$g_defaultOptionValues["addPageId"] = "";
	$g_defaultOptionValues["LinkField"] = "";
	$g_defaultOptionValues["LinkFieldType"] = 0;
	$g_defaultOptionValues["LinkPrefix"] = "";
	$g_defaultOptionValues["list"] = false;
	$g_defaultOptionValues["listAjax"] = false;

	$g_defaultOptionValues["locking"] = false;
	$g_defaultOptionValues["LookupDesc"] = false;
	$g_defaultOptionValues["LookupOrderBy"] = "";
	$g_defaultOptionValues["LookupTable"] = "";
	$g_defaultOptionValues["LookupType"] = 0;
	$g_defaultOptionValues["LookupUnique"] = false;
	$g_defaultOptionValues["LookupValues"] = array();
	$g_defaultOptionValues["LookupWhere"] = "";
	$g_defaultOptionValues["LookupWhereCode"] = false;
//	M
	$g_defaultOptionValues["mainTableOwnerID"] = "";
	$g_defaultOptionValues["mapData"] = array();
	$g_defaultOptionValues["masterListFields"] = array();
	$g_defaultOptionValues["maxFileSize"] = null;
	$g_defaultOptionValues["maxImageHeight"] = null;
	$g_defaultOptionValues["maxImageWidth"] = null;
	$g_defaultOptionValues["maxNumberOfFiles"] = 1;
	$g_defaultOptionValues["maxTotalFilesSize"] = null;
	$g_defaultOptionValues["moveNext"] = "";
	$g_defaultOptionValues["Multiselect"] = false;
//	N
	$g_defaultOptionValues["NCSearch"] = false;
	$g_defaultOptionValues["NeedEncode"] = false;
	$g_defaultOptionValues["NewSize"] = 0;
	$g_defaultOptionValues["noRecordsFirstPage"] = false;
	$g_defaultOptionValues["nSecOptions"] = ADVSECURITY_NONE;
	$g_defaultOptionValues["NumberOfChars"] = 0;
	$g_defaultOptionValues["numberOfVisibleItems"] = 10;
	$g_defaultOptionValues["nViewPagePDFScale"] = 100;
	$g_defaultOptionValues["nPrinterPagePDFScale"] = 100;
	$g_defaultOptionValues["nPrinterPageScale"] = 100;
	$g_defaultOptionValues["nPrinterPDFSplitRecords"] = 0;
	$g_defaultOptionValues["nPrinterSplitRecords"] = 0;


//	O
	$g_defaultOptionValues["OriginalTable"] = "";
	$g_defaultOptionValues["orderindexes"] = array();
	$g_defaultOptionValues["OwnerID"] = 0;
	$g_defaultOptionValues["ownerTable"] = "";
	$g_defaultOptionValues["OraSequenceName"] = "";
//	P
	$g_defaultOptionValues["pageSize"] = 0;
	$g_defaultOptionValues["panelSearchFields"] = array();
	$g_defaultOptionValues["parentFilterField"] = "";
	$g_defaultOptionValues["printFriendly"] = false;
	$g_defaultOptionValues["printFields"] = array();
	$g_defaultOptionValues["printGridLayout"] = 0;
	$g_defaultOptionValues["printReportLayout"] = 0;
	$g_defaultOptionValues["printerPageOrientation"] = 0;
//	Q
//	R
	$g_defaultOptionValues["recsLimit"] = 0;
	$g_defaultOptionValues["recsPerRowList"] = 1;
	$g_defaultOptionValues["recsPerRowPrint"] = 1;

	$g_defaultOptionValues["ResizeImage"] = false;
	$g_defaultOptionValues["RewindEnabled"] = false;
	$g_defaultOptionValues["rowHighlite"] = false;
	$g_defaultOptionValues["requiredSearchFields"] = array();
	$g_defaultOptionValues["reportPrintGroupsPerPage"] = 3;
	$g_defaultOptionValues["reportPrintPDFGroupsPerPage"] = 0;

//	S
	$g_defaultOptionValues["searchFields"] = false;
	$g_defaultOptionValues["searchPanelOptions"] = array();
	$g_defaultOptionValues["SelectSize"] = 1;
	$g_defaultOptionValues["selectExportDelimiter"] = false;
	$g_defaultOptionValues["selectExportFields"] = false;
	$g_defaultOptionValues["searchIsRequiredForFilters"] = false;
	$g_defaultOptionValues["searchOptionsList"] = array();
	$g_defaultOptionValues["ShortName"] = "";
	$g_defaultOptionValues["shortTableName"] = "";
	$g_defaultOptionValues["showAddInPopup"] = false;
	$g_defaultOptionValues["ShowCustomExpr"] = false;
	$g_defaultOptionValues["showEditInPopup"] = false;
	$g_defaultOptionValues["ShowFileSize"] = false;
	$g_defaultOptionValues["ShowIcon"] = false;
	$g_defaultOptionValues["showSearchPanel"] = false;
	$g_defaultOptionValues["showSimpleSearchOptions"] = false;
	$g_defaultOptionValues["ShowThumbnail"] = false;
	$g_defaultOptionValues["ShowTime"] = false;
	$g_defaultOptionValues["showViewInPopup"] = false;
	$g_defaultOptionValues["showCollapsed"] = false;
	$g_defaultOptionValues["showWithNoRecords"] = false;
	$g_defaultOptionValues["SimpleAdd"] = false;
	$g_defaultOptionValues["ShowListOfThumbnails"] = false;
	$g_defaultOptionValues["sortValueType"] = SORT_BY_DISP_VALUE;
	$g_defaultOptionValues["sqlFrom"] = "";
	$g_defaultOptionValues["sqlHead"] = "";
	$g_defaultOptionValues["sqlTail"] = "";
	$g_defaultOptionValues["sqlWhereExpr"] = "";
	$g_defaultOptionValues["sqlquery"] = null;
	$g_defaultOptionValues["strField"] = "";
	$g_defaultOptionValues["strFilename"] = "";
	$g_defaultOptionValues["strName"] = "";
	$g_defaultOptionValues["strOrderBy"] = "";
	$g_defaultOptionValues["StrThumbnail"] = "";
	$g_defaultOptionValues["strSortControlSettingsJSON"] = "";
	$g_defaultOptionValues["strClickActionJSON"] = "";
	$g_defaultOptionValues["ThumbnailSize"] = 0;
	$g_defaultOptionValues["scrollGridBody"] = false;
//	T
	$g_defaultOptionValues["totalsFields"] = array();
	$g_defaultOptionValues["ThumbHeight"] = 72;
	$g_defaultOptionValues["ThumbWidth"] = 72;
	$g_defaultOptionValues["truncateText"] = false;
//	U
	$g_defaultOptionValues["UploadFolder"] = "";
	$g_defaultOptionValues["UploadCodeExpression"] = false;
	$g_defaultOptionValues["UseCategory"] = false;
	$g_defaultOptionValues["UseRTE"] = false;
	$g_defaultOptionValues["UseTimestamp"] = false;
//	V
	$g_defaultOptionValues["validateAs"] = array();
	$g_defaultOptionValues["videoHeight"] = 200;
	$g_defaultOptionValues["videoWidth"] = 300;
	$g_defaultOptionValues["videoTitleField"] = "";
	$g_defaultOptionValues["view"] = false;
	$g_defaultOptionValues["ViewFormat"] = "";
//	W
	$g_defaultOptionValues["warnLeavingPages"] = false;
	$g_defaultOptionValues["weekdays"] = "";
	$g_defaultOptionValues["weekdayMessage"] = array();
//	X
//	Y
//	Z
$g_defaultOptionValues["groupChart"] = false;
$g_defaultOptionValues["chartLabelInterval"] = 0;
$g_defaultOptionValues["chartSeries"] = array();



//	A
	$g_settingsType["Absolute"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["acceptFileTypes"] = SETTING_TYPE_EDIT;
	$g_settingsType["acceptFileTypesHtml"] = SETTING_TYPE_EDIT;
	$g_settingsType["AllowToAdd"] = SETTING_TYPE_EDIT;
	$g_settingsType["autoCompleteFields"] = SETTING_TYPE_EDIT;
	$g_settingsType["autoCompleteFieldsOnEdit"] = SETTING_TYPE_EDIT;
	$g_settingsType["AutoInc"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["audioTitleField"] = SETTING_TYPE_VIEW;
	$g_settingsType["autoUpload"] = SETTING_TYPE_EDIT;
	$g_settingsType["autoUpdatable"] = SETTING_TYPE_EDIT;
//	B
	$g_settingsType["bInlineAdd"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bInlineEdit"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bUpdateSelected"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bIsEncrypted"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bListPage"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["bPrinterPage"] = SETTING_TYPE_GLOBAL;
//	C
	$g_settingsType["CompatibilityMode"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["categoryFields"] = SETTING_TYPE_EDIT;
	$g_settingsType["controlWidth"] = SETTING_TYPE_EDIT;
	$g_settingsType["CreateThumbnail"] = SETTING_TYPE_EDIT;
	$g_settingsType["CustomDisplay"] = SETTING_TYPE_EDIT;
//	D
	$g_settingsType["DateEditType"] = SETTING_TYPE_EDIT;
	$g_settingsType["DecimalDigits"] = SETTING_TYPE_VIEW;
	$g_settingsType["DeleteAssociatedFile"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["denyDuplicates"] = SETTING_TYPE_EDIT;
	$g_settingsType["DependentLookups"] = SETTING_TYPE_EDIT;
	$g_settingsType["DisplayField"] = SETTING_TYPE_EDIT;
//	E
	$g_settingsType["EditFormat"] = SETTING_TYPE_EDIT;
	$g_settingsType["EditParams"] = SETTING_TYPE_EDIT;
//	F
	$g_settingsType["fieldIsVideoUrl"] = SETTING_TYPE_VIEW;
	$g_settingsType["fieldIsImageUrl"] = SETTING_TYPE_VIEW;
	$g_settingsType["FieldType"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["FieldPermissions"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["Filename"] = SETTING_TYPE_VIEW;
	$g_settingsType["FormatTimeAttrs"] = SETTING_TYPE_EDIT;
	$g_settingsType["freeInput"] = SETTING_TYPE_EDIT;
	$g_settingsType["FullName"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["fieldEvents"] = SETTING_TYPE_EDIT;
	$g_settingsType["fieldViewEvents"] = SETTING_TYPE_VIEW;
//	G
	$g_settingsType["GoodName"] = SETTING_TYPE_GLOBAL;
//	H
	$g_settingsType["HorizontalLookup"] = SETTING_TYPE_EDIT;
	$g_settingsType["hlNewWindow"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlType"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlLinkWordNameType"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlLinkWordText"] = SETTING_TYPE_VIEW;
	$g_settingsType["hlTitleField"] = SETTING_TYPE_VIEW;
	$g_settingsType["HTML5InuptType"] = SETTING_TYPE_EDIT;
//	I
	$g_settingsType["Index"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["InitialYearFactor"] = SETTING_TYPE_EDIT;
	$g_settingsType["ImageHeight"] = SETTING_TYPE_VIEW;
	$g_settingsType["ImageWidth"] = SETTING_TYPE_VIEW;
	$g_settingsType["insertNull"] = SETTING_TYPE_EDIT;
	$g_settingsType["IsRequired"] = SETTING_TYPE_EDIT;
	$g_settingsType["isSeparate"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["UpdateLatLng"] = SETTING_TYPE_EDIT;
//	J
//	K
//	L
	$g_settingsType["Label"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["LastYearFactor"] = SETTING_TYPE_EDIT;
	$g_settingsType["LCType"] = SETTING_TYPE_EDIT;
	$g_settingsType["listPageId"] = SETTING_TYPE_EDIT;
	$g_settingsType["addPageId"] = SETTING_TYPE_EDIT;
	$g_settingsType["LinkField"] = SETTING_TYPE_EDIT;
	$g_settingsType["LinkFieldType"] = SETTING_TYPE_EDIT;
	$g_settingsType["LinkPrefix"] = SETTING_TYPE_VIEW;
	$g_settingsType["LookupConnId"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupDesc"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupOrderBy"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupTable"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupType"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupUnique"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupValues"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupWhere"] = SETTING_TYPE_EDIT;
	$g_settingsType["LookupWhereCode"] = SETTING_TYPE_EDIT;
//	M
	$g_settingsType["mapData"] = SETTING_TYPE_VIEW;
	$g_settingsType["maxFileSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxImageHeight"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxImageWidth"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxNumberOfFiles"] = SETTING_TYPE_EDIT;
	$g_settingsType["maxTotalFilesSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["Multiselect"] = SETTING_TYPE_EDIT;
//	N
	$g_settingsType["nCols"] = SETTING_TYPE_EDIT;
	$g_settingsType["NeedEncode"] = SETTING_TYPE_VIEW;
	$g_settingsType["NewSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["nRows"] = SETTING_TYPE_EDIT;
	$g_settingsType["nSecOptions"] = ADVSECURITY_NONE;
	$g_settingsType["NumberOfChars"] = SETTING_TYPE_VIEW;
//	O
	$g_settingsType["ownerTable"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["OraSequenceName"] = SETTING_TYPE_GLOBAL;
//	P
//	Q
//	R
	$g_settingsType["ResizeImage"] = SETTING_TYPE_EDIT;
	$g_settingsType["RewindEnabled"] = SETTING_TYPE_VIEW;
//	S
	$g_settingsType["SelectSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["ShowCustomExpr"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowFileSize"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowIcon"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowThumbnail"] = SETTING_TYPE_VIEW;
	$g_settingsType["ShowTime"] = SETTING_TYPE_EDIT;
	$g_settingsType["SimpleAdd"] = SETTING_TYPE_EDIT;
	$g_settingsType["ShowListOfThumbnails"] = SETTING_TYPE_VIEW;
	$g_settingsType["strField"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["strFilename"] = SETTING_TYPE_VIEW;
	$g_settingsType["filterTotalFields"] = '';
	$g_settingsType["strEditMask"] = SETTING_TYPE_EDIT;
	$g_settingsType["strName"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["StrThumbnail"] = SETTING_TYPE_EDIT;
//	T
	$g_settingsType["ThumbnailSize"] = SETTING_TYPE_EDIT;
	$g_settingsType["ThumbHeight"] = SETTING_TYPE_VIEW;
	$g_settingsType["ThumbWidth"] = SETTING_TYPE_VIEW;
	$g_settingsType["truncateText"] = SETTING_TYPE_VIEW;
	$g_settingsType["timeFormatData"] = SETTING_TYPE_VIEW;
//	U
	$g_settingsType["UploadFolder"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["UploadCodeExpression"] = SETTING_TYPE_GLOBAL;
	$g_settingsType["UseCategory"] = SETTING_TYPE_EDIT;
	$g_settingsType["UseRTE"] = SETTING_TYPE_EDIT;
	$g_settingsType["UseTimestamp"] = SETTING_TYPE_EDIT;
//	V
	$g_settingsType["validateAs"] = SETTING_TYPE_EDIT;
	$g_settingsType["videoHeight"] = SETTING_TYPE_VIEW;
	$g_settingsType["videoWidth"] = SETTING_TYPE_VIEW;
	$g_settingsType["videoTitleField"] = SETTING_TYPE_VIEW;
	$g_settingsType["ViewFormat"] = SETTING_TYPE_VIEW;

	$g_settingsType["multipleImgMode"] = SETTING_TYPE_VIEW;
	$g_settingsType["maxImages"] = SETTING_TYPE_VIEW;
	$g_settingsType["showGallery"] = SETTING_TYPE_VIEW;
	$g_settingsType["galleryMode"] = SETTING_TYPE_VIEW;
	$g_settingsType["captionMode"] = SETTING_TYPE_VIEW;
	$g_settingsType["captionField"] = SETTING_TYPE_VIEW;
	$g_settingsType["imageBorder"] = SETTING_TYPE_VIEW;
	$g_settingsType["imageFullWidth"] = SETTING_TYPE_VIEW;

	$g_settingsType["weekdays"] = SETTING_TYPE_EDIT;
	$g_settingsType["weekdayMessage"] = SETTING_TYPE_EDIT;

?>